import os, sys

import numpy, h5py
from PyQt5.QtCore import QRect
from PyQt5.QtWidgets import QApplication, QMessageBox, QFileDialog

from orangewidget import gui
from orangewidget.settings import Setting

from oasys.widgets.widget import OWWidget
from oasys.widgets.widget import OWWidget
from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence

from orangecontrib.esrf.util.dabam2d_util import scan_root_directory
from silx.gui.plot import Plot2D
from scipy.interpolate import  RectBivariateSpline
from srxraylib.metrology.profiles_simulation import slopes
import oasys.util.oasys_util as OU
from oasys.util.oasys_objects import OasysSurfaceData

class OWdabam2d(OWWidget):
    name = "DABAM2D File Access and Processing"
    id = "dabam2d_file_access_and_processing"
    description = "DABAM2D File Access and Processing"
    icon = "icons/dabam2d.png"
    author = "M Sanchez del Rio"
    maintainer_email = "srio@esrf.eu"
    priority = 100
    category = ""
    keywords = ["dabam2d_file_access_and_processing"]

    outputs = [{"name": "Surface Data",
                "type": OasysSurfaceData,
                "doc": "Surface Data",
                "id": "Surface Data"},
               {"name": "DABAM 1D Profile",
                "type": numpy.ndarray,
                "doc": "numpy.ndarray",
                "id": "numpy.ndarray"},
               ]

    want_main_area = 1
    want_control_area = 1

    MAX_WIDTH = 1320
    MAX_HEIGHT = 700

    IMAGE_WIDTH = 800
    IMAGE_HEIGHT = 610

    CONTROL_AREA_WIDTH = 405

    #
    # variables stored in the widget
    #
    conversion_to_m_z = Setting(1.0)
    root = Setting("<None>")
    file_out = Setting("dabam2d_tmp.h5")
    rebase = Setting(0)
    extract_profile1D = Setting(0)
    coordinate_profile1D = Setting(0.0)
    #
    # local variables
    #
    data=None
    scanned_files_with_path = []
    scanned_files_without_path = []
    selected_files_with_path = []
    selected_files_without_path = []

    def __init__(self):
        super().__init__()

        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width() * 0.05),
                               round(geom.height() * 0.05),
                               round(min(geom.width() * 0.98, self.MAX_WIDTH)),
                               round(min(geom.height() * 0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())

        gui.separator(self.controlArea)

        # main buttons
        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")
        button = gui.button(button_box, self, "Write and Send (sum of) Selection", callback=self.write_and_send)
        button.setFixedHeight(45)

        #
        # input file list
        #
        input_box_l = oasysgui.widgetBox(self.controlArea, "Input Dabax2d files", addSpace=True, orientation="vertical", height=460, width=self.CONTROL_AREA_WIDTH)

        root_file_box = oasysgui.widgetBox(input_box_l, "", addSpace=True, orientation="vertical",)
        figure_box = oasysgui.widgetBox(root_file_box, "", addSpace=True, orientation="horizontal")
        oasysgui.lineEdit(figure_box, self, "root", "root dir:",
                                                    labelWidth=80, valueType=str, orientation="horizontal")
        gui.button(figure_box, self, "...", callback=self.select_root)

        gui.button(input_box_l, self, "Scan root directory", callback=self.scan_directory)

        self.files_area = oasysgui.textArea(height=250)
        self.refresh_files_text_area()
        input_box_l.layout().addWidget(self.files_area)

        # operations
        operations_box = oasysgui.widgetBox(input_box_l, "Operations", addSpace=False, orientation="vertical")
        oasysgui.lineEdit(operations_box, self, "conversion_to_m_z", label="Scaling factor", labelWidth=300, orientation="horizontal", valueType=float)
        gui.comboBox(operations_box, self, "rebase", label="Set min to zero", labelWidth=220,
                     items=["No",
                            "Yes",],
                     sendSelectedValue=False, orientation="horizontal")

        # buttons
        button_box_2 = oasysgui.widgetBox(input_box_l, "", addSpace=False, orientation="horizontal")
        gui.button(button_box_2, self, "View Selection", callback=self.view_selection)
        gui.button(button_box_2, self, "View Sum of Selection", callback=self.view_sum)
        gui.button(button_box_2, self, "Clear Views", callback=self.clear_views)

        # write and send
        write_and_send_box = oasysgui.widgetBox(self.controlArea, "Write and Send", addSpace=True,
                                         orientation="vertical",)

        oasysgui.lineEdit(write_and_send_box, self, "file_out", "output file:",
                                                    labelWidth=80, valueType=str, orientation="horizontal")

        gui.comboBox(write_and_send_box, self, "extract_profile1D", label="Extract and send 1D profile", labelWidth=220,
                     items=["axis 0 (horizontal)", "axis 1 (vertical)"],
                     sendSelectedValue=False, orientation="horizontal")

        oasysgui.lineEdit(write_and_send_box, self, "coordinate_profile1D", "At coordinate [m]:", labelWidth=260, valueType=float,
                          orientation="horizontal")

        # gui.separator(tab_out, height=20)

        # results
        main_tabs = oasysgui.tabWidget(self.mainArea)

        plot_tab = oasysgui.createTabPage(main_tabs, "Surfaces")


        self.tab = []
        self.tabs = oasysgui.tabWidget(plot_tab)

        self.clear_views()
        self.append_tabs_surfaces()

        gui.rubber(self.controlArea)
        gui.rubber(self.mainArea)

    def delete_tab(self):
        """
        main_tabs.setTabsClosable(True)
        main_tabs.tabCloseRequested.connect(self.delete_tab)
        """
        print(">>>> Delete tab")

    def select_root(self):
        root = QFileDialog.getExistingDirectory(self, "Select root/url directory to scan for h5 files", "")
        if root != "":
            self.root = root

    def write_and_send(self):
        self.get_selection()
        self.read_selected_data_files_sum()

        # self.zernike()

        # self.write_thickness_file()

        zz = numpy.round(self.data[-1][2], 12)
        xx = numpy.round(self.data[-1][0], 12)
        yy = numpy.round(self.data[-1][1], 12)
        filename = self.file_out
        try:
            OU.write_surface_file(zz.T, xx, yy, file_name=filename)
            print("File %s written to disk." % filename)
        except:
                print("*** Error writing hdf5 file **")

        # send 2D profile
        xx = self.data[-1][0]
        yy = self.data[-1][1]
        zz = self.data[-1][2]
        self.send("Surface Data",
                  OasysSurfaceData(xx=xx,
                                   yy=yy,
                                   zz=zz.T,
                                   surface_data_file=self.file_out))
        # send 1D profile
        # self.extract_profile1D = 0
        if self.extract_profile1D == 0:
            abscissas = xx
            perp_abscissas = yy
            index0 = numpy.argwhere(perp_abscissas >= 0.0)
            try:
                index0 = index0[0][0]
            except:
                index0 = -1
            profile1D = zz[:, index0]
            # slope1D = numpy.gradient(profile1D, abscissas)
            # profile1D_std = profile1D.std()
            # slope1D_std = slope1D.std()
            # if self.invert_axes_names:
            #     title = "profile at X[%d] = %g; StDev = %g um" % (index0, perp_abscissas[index0], 1e6*profile1D_std)
            #     titleS = "slopes at X[%d] = %g; StDev = %g urad" % (index0, perp_abscissas[index0], 1e6*slope1D_std)
            #     xtitle = "Y [m] "
            # else:
            #     title = "profile at Y[%d] = %g; StDev = %g um" % (index0, perp_abscissas[index0], 1e6*profile1D_std)
            #     titleS = "slopes at Y[%d] = %g; StDev = %g urad" % (index0, perp_abscissas[index0], 1e6*slope1D_std)
            #     xtitle = "X [m] "
            # self.plot_data1D(abscissas, 1e6*profile1D, self.profile1D_id, title=title, xtitle=xtitle, ytitle="Z [um] ")
            # self.plot_data1D(abscissas, 1e6*slope1D, self.slope1D_id, title=titleS, xtitle=xtitle, ytitle="Z' [urad]")
        else:
            abscissas = yy
            perp_abscissas = xx
            index0 = numpy.argwhere(perp_abscissas >= self.coordinate_profile1D)
            try:
                index0 = index0[0][0]
            except:
                index0 = -1
            profile1D = zz[index0, :]
            # slope1D = numpy.gradient(profile1D,abscissas)
            # profile1D_std = profile1D.std()
            # slope1D_std = slope1D.std()
            # if self.invert_axes_names:
            #     title = "profile at Y[%d] = %g; StDev = %g um" % (index0, perp_abscissas[index0], 1e6*profile1D_std)
            #     titleS = "slopes at Y[%d] = %g; StDev = %g urad" % (index0, perp_abscissas[index0], 1e6*slope1D_std)
            #     xtitle = "X [m] "
            # else:
            #     title = "profile at X[%d] = %g; StDev = %g um" % (index0, perp_abscissas[index0], 1e6*profile1D_std)
            #     titleS = "slopes at X[%d] = %g; StDev = %g urad" % (index0, perp_abscissas[index0], 1e6*slope1D_std)
            #     xtitle = "Y [m] "
            # self.plot_data1D(abscissas, 1e6*profile1D, self.profile1D_id, title=title, xtitle=xtitle, ytitle="Z [um] ")
            # self.plot_data1D(abscissas, 1e6*slope1D, self.slope1D_id, title=titleS, xtitle=xtitle, ytitle="Z' [urad]")

        dabam_profile = numpy.zeros((profile1D.size, 2))
        dabam_profile[:, 0] = abscissas
        dabam_profile[:, 1] = profile1D
        # from srxraylib.plot.gol import plot
        # plot(abscissas, profile1D)
        self.send("DABAM 1D Profile", dabam_profile)

    def scan_directory(self):
        scanned_files_without_path, scanned_files_with_path  = scan_root_directory(self.root)
        self.scanned_files_with_path = scanned_files_with_path
        self.scanned_files_without_path = scanned_files_without_path
        self.refresh_files_text_area(scanned_files_without_path)

    def refresh_files_text_area(self, files=None):
        text = ""
        if files is not None:
            for file in files:
                text += file + "\n"
        self.files_area.setText(text)

    # def write_thickness_file(self):
    #     zz = numpy.round(self.data[-1][2], 12)
    #     xx = numpy.round(self.data[-1][0], 12)
    #     yy = numpy.round(self.data[-1][1], 12)
    #     filename = self.file_out
    #     try:
    #         OU.write_surface_file(zz, xx, yy, file_name=filename)
    #         print("File %s written to disk." % filename)
    #     except:
    #         print("*** Error writing hdf5 file **")

    def clear_views(self):
        size = len(self.tab)
        for index in range(size):
            self.tabs.removeTab(size-1-index)
        self.tab = []

    def append_tabs_surfaces(self):
        current_tab = self.tabs.currentIndex()

        files = []
        for surface_file_name in self.selected_files_with_path:
            files.append(os.path.basename(surface_file_name))

        #
        if not len(files) == 0:
            for title in files:
                self.tab.append(oasysgui.createTabPage(self.tabs, title))

            for tab in self.tab:
                tab.setFixedHeight(self.IMAGE_HEIGHT)
                tab.setFixedWidth(self.IMAGE_WIDTH)

            self.tabs.setCurrentIndex(current_tab)
        else:
            pass

    def get_selection(self):
        cursor = self.files_area.textCursor()
        start = cursor.selectionStart()
        end = cursor.selectionEnd()
        print('Selected Count: ' + str(end - start))
        txt = self.files_area.textCursor().selectedText()
        selected_files = txt.split("\u2029")
        print('Selected files:\n ', selected_files)

        IDX = []
        for file in selected_files:
            if file in self.scanned_files_without_path:
                idx = self.scanned_files_without_path.index(file)
                IDX.append(idx)
                print("** found: ", file, idx)

        print("IDX: ", IDX)

        SELECTED_FILES_WITH_PATH = []
        SELECTED_FILES_WITHOUT_PATH = []
        for idx in IDX:
            SELECTED_FILES_WITH_PATH.append(self.scanned_files_with_path[idx])
            SELECTED_FILES_WITHOUT_PATH.append(self.scanned_files_without_path[idx])

        if len(IDX) > 0: print("SELECTED_PATH", idx, SELECTED_FILES_WITH_PATH)

        self.selected_files_with_path = SELECTED_FILES_WITH_PATH
        self.selected_files_without_path = SELECTED_FILES_WITHOUT_PATH

    def view_selection(self):
        self.get_selection()

        try:
            self.read_selected_data_files()
            self.append_tabs_surfaces()
            self.plot_selection()
        except Exception as exception:
            QMessageBox.critical(self, "Error",
                                 exception.args[0],
                                 QMessageBox.Ok)

            if self.IS_DEVELOP: raise exception

    def view_sum(self):
        self.get_selection()

        try:
            self.read_selected_data_files_sum()
            # self.append_tabs_surfaces()
            current_tab = self.tabs.currentIndex()
            self.tab.append(oasysgui.createTabPage(self.tabs, "sum"))
            for tab in self.tab:
                tab.setFixedHeight(self.IMAGE_HEIGHT)
                tab.setFixedWidth(self.IMAGE_WIDTH)

            self.tabs.setCurrentIndex(current_tab)
            self.plot_selection(title="sum")

        except Exception as exception:
            QMessageBox.critical(self, "Error",
                                 exception.args[0],
                                 QMessageBox.Ok)

            if self.IS_DEVELOP: raise exception

    def read_selected_data_files(self):
        self.data = []
        for surface_file_name in self.selected_files_with_path:
            surface_file_name = congruence.checkDir(surface_file_name)

            try: # OASYS surface data format
                file = h5py.File(surface_file_name, 'r')
                zz = file["surface_file" + "/Z"][()].T
                xx = file["surface_file" + "/X"][()]
                yy = file["surface_file" + "/Y"][()]
                print("Data read from file: ", surface_file_name, zz.shape, xx.shape, yy.shape)
            except Exception as ee:
                raise IOError("Error loading HDF5 file" + str(ee))

            zz = zz * self.conversion_to_m_z # if self.negate == 0 else -1.0 * zz * self.conversion_to_m_z
            if self.rebase: zz -= zz.min()

            self.data.append([xx, yy, zz])

    def read_selected_data_files_sum(self):

        for i, surface_file_name in enumerate(self.selected_files_with_path):
            surface_file_name = congruence.checkDir(surface_file_name)

            try: # OASYS surface data format
                file = h5py.File(surface_file_name, 'r')
                zz = file["surface_file" + "/Z"][()].T
                xx = file["surface_file" + "/X"][()]
                yy = file["surface_file" + "/Y"][()]
                print("Data read from file: ", surface_file_name, zz.shape, xx.shape, yy.shape)
            except Exception as ee:
                raise IOError("Error loading HDF5 file" + str(ee))

            zz = zz * self.conversion_to_m_z # if self.negate == 0 else -1.0 * zz * self.conversion_to_m_z
            if self.rebase: zz -= zz.min()

            if i == 0:
                XX = xx.copy()
                YY = yy.copy()
                ZZ = zz.copy()
            else:
                interpolator = RectBivariateSpline(xx, yy, zz, kx=3, ky=3, s=0)
                zz_interpolated = interpolator(XX, YY)
                print(">>>> adding index orig interpolated ", i, zz.shape, zz_interpolated.shape, surface_file_name)
                print(">>>> adding to ", ZZ.shape)
                ZZ += zz_interpolated

        self.data = []
        try:
            self.data.append([XX, YY, ZZ])
        except:
            pass

    def plot_data2D(self, data2D, dataX, dataY, tabs_canvas_index, title="title", xtitle="X",ytitle="Y"):

        print(">>>>>> plotting ", data2D.shape, dataX.shape, dataY.shape)
        tabs_canvas_index.layout().removeItem(tabs_canvas_index.layout().itemAt(0))

        origin = (dataX[0], dataY[0])
        scale = (dataX[1] - dataX[0], dataY[1] - dataY[0])

        colormap = {"name": "temperature", "normalization": "linear",
                    "autoscale": True, "vmin": 0, "vmax": 0, "colors": 256}

        tmp = Plot2D()
        tmp.resetZoom()
        tmp.setXAxisAutoScale(True)
        tmp.setYAxisAutoScale(True)
        tmp.setGraphGrid(False)
        tmp.setKeepDataAspectRatio(True)
        tmp.yAxisInvertedAction.setVisible(False)
        tmp.setXAxisLogarithmic(False)
        tmp.setYAxisLogarithmic(False)
        tmp.getMaskAction().setVisible(False)
        tmp.getRoiAction().setVisible(False)
        tmp.getColormapAction().setVisible(True)
        tmp.setKeepDataAspectRatio(False)
        tmp.addImage(data2D.T,legend="1",scale=scale,origin=origin,colormap=colormap,replace=True)
        tmp.setActiveImage("1")
        tmp.setGraphXLabel(xtitle)
        tmp.setGraphYLabel(ytitle)
        tmp.setGraphTitle(title)
        tabs_canvas_index.layout().addWidget(tmp)

    def plot_selection(self, title=None):
        #
        # result
        #
        print(">>>>>>> in plot selection len: ", len(self.data))
        for index in range(len(self.data)):
            tab_index = -len(self.data) + index
            print("plotting fig index ", index, "in tab_index", tab_index)

            xx, yy, zz = self.data[index]
            title_top = "file: %s" % self.selected_files_without_path[index]
            if len(self.selected_files_without_path) > 1 and title is not None: title_top=title
            print("title, title_top: ", title, title_top, len(self.selected_files_without_path))

            self.plot_data2D(zz, xx, yy,
                       self.tab[tab_index],
                       title=title_top,
                       xtitle="X [m] (%d pixels, max:%f)"%(xx.size, xx.max()),
                       ytitle="Y [m] (%d pixels, max:%f)"%(yy.size, yy.max()) )


    def zernike(self):
        ################################### ZERNIKE #################################################################
        import barc4ro.barc4ro as b4RO
        N = 37
        heightProfData = self.data[-1][2].T
        Zcoeffs, fit, residues = b4RO.fit_zernike_circ(heightProfData, nmodes=N, startmode=1, rec_zern=False)
        print('Zernike coefficients (um): \n' + str(Zcoeffs * 1e6))

        from barc4plots.refactored_barc4plots import Data4Plot
        p = Data4Plot(image = Zcoeffs * 1e6, axis_x = None, axis_y = None, sort_ax = False, sort_ax_lim = False)
        p.label = "ZZZ"
        p.AxLegends[0] = "top title"
        p.AxLegends[1] = "coefficients"
        p.AxLegends[2] = r"$\mu$m"
        p.MinMax[0] = -2.5
        p.MinMax[1] = 2.5
        p.plot_wft_coeffs(noll=True)
        ################################### END ZERNIKE #############################################################
    def print_statistics(self):
        for data_i in self.data:

            slopeErrorRMS_X, slopeErrorRMS_Y = slopes(data_i[2],
                         data_i[0],
                         data_i[1],
                         silent=1, return_only_rms=1)


            print("\n*****************")
            print("Heigh error StDev: %g um" % (1e6 * data_i[2].std()))
            print("Slope error StDev in axis0: %g urad" % (1e6 * slopeErrorRMS_X))
            print("Slope error StDev in axis1: %g urad" % (1e6 * slopeErrorRMS_Y))
            print("*****************")

            import barc4ro.barc4ro as b4RO
            Zcoeffs, fit, residues = b4RO.fit_zernike_circ(data_i[2], nmodes=37, startmode=1, rec_zern=False)
            print('Zernike coefficients (um): \n' + str(Zcoeffs * 1e6))
if __name__ == "__main__":
    if 1:
        app = QApplication(sys.argv)
        w = OWdabam2d()
        w.root = "/nobackup/gurb1/srio/DABAM2D/ESRF/C_2D_R320um"
        w.show()
        app.exec()
        w.saveSettings()
    else:
        # import re
        # import urllib
        #
        # # def ls_ows(target_url="https://github.com/oasys-esrf-kit/dabam2d/tree/main/data/"):
        # def ls_ows(target_url="https://github.com/PaNOSC-ViNYL/Oasys-PaNOSC-Workspaces"):
        #     data = urllib.request.urlopen(target_url)
        #     list_files = []
        #     for line in data:
        #         line1 = str(line)
        #
        #         if ".dat" in line1:
        #             try:
        #                 found = re.search('dat">(.+?)</a>', line1).group(1)
        #                 list_files.append(found)
        #             except:
        #                 pass
        #
        # print(ls_ows("https://github.com/oasys-kit/dabaxFiles"))
        import github

        g = github.Github("USERNAME", "PASSWORD")
        repo = g.get_user().get_repo("REPO_NAME")
        print
        repo.get_dir_contents("")



import numpy
import os
import pathlib
import glob


def get_directory_contents(directory):
    dirnames = []
    filenames = []
    for (dirpath, dirnames, filenames) in os.walk(directory):
        # level = dirpath.replace(directory, '').count(os.sep)
        # indent = ' ' * 4 * (level)
        # print('{}{}/'.format(indent, os.path.basename(dirpath)))
        # subindent = ' ' * 4 * (level + 1)
        # for f in filenames:
        #     print('{}{}'.format(subindent, f))
        break
    return dirnames, filenames

def scan_root_directory(directory, search_for='.h5'):

    # LENS = []
    # FAMILY = []
    PATH = []
    FILE = []

    for path, dirs, files in os.walk(directory):
        for f in files:
            file_with_path = path + os.sep + f
            if search_for in file_with_path:
                PATH.append(file_with_path)
                items = file_with_path.split(os.sep)
                FILE.append(items[-1])
                # LENS.append(items[-3])
                # FAMILY.append(items[-4])
    return FILE, PATH

if __name__ == "__main__":
    # LENS, FAMILY, PATH = scan_root_directory("/nobackup/gurb1/srio/DABAM2D/ESRF")
    #
    # for i in range(len(PATH)):
    #     print(">>>", FAMILY[i], LENS[i])

    LENS, FAMILY, FILE, PATH = scan_root_directory("/nobackup/gurb1/srio/DABAM2D/ESRF/C_2D_R320um")


    # for path, dirs, files in os.walk("/nobackup/gurb1/srio/DABAM2D/ESRF"):
    #     # print (">>>>", path)
    #     for f in files:
    #         file_with_path = path + os.sep + f
    #         if '.h5' in file_with_path:
    #             PATH.append(file_with_path)
    #             items = file_with_path.split(os.sep)
    #             LENS.append(items[-3])
    #             FAMILY.append(items[-4])

    for i in range(len(PATH)):
        print(">>>", FAMILY[i], LENS[i], FILE[i]) #,  PATH[i])

    print("Total h5 files found: ", len(FILE))


import os, sys
import numpy
from PyQt5.QtWidgets import QApplication, QMessageBox, QSizePolicy

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui, congruence
from oasys.widgets.exchange import DataExchangeObject

from oasys.widgets.exchange import DataExchangeObject
from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget

import scipy.constants as codata


class OWBM05Calculator(XoppyWidget):
    name = "BM05_Calculator"
    id = "orange.widgets.dataxpower"
    description = "Calculates some values used for BM05"
    icon = "icons/bm05_calculator.png"
    priority = 1
    category = ""
    keywords = ["xoppy", "power"]

    inputs = [("ExchangeData", DataExchangeObject, "acceptExchangeData")]

    SOURCE = Setting(2)
    ENER_MIN = Setting(1000.0)
    ENER_MAX = Setting(50000.0)
    ENER_N = Setting(100)
    SOURCE_FILE = Setting("?")
    
    BM05_CONFIGURATION = Setting(3)   
    CHOPPER = Setting(100.0)    
    EXTRA_AIR_BEFORE_SAMPLE = Setting(0.0)
    CALCULATION_TYPE = Setting(0)
    PROPAGATION_DISTANCE = Setting(10)
    SCINTILLATOR_EFFICIENCY = Setting(0) #ph/keV
            
    input_spectrum = None
    input_script = None
    output_file = None
    

    def __init__(self):
        
        super().__init__(show_script_tab=True)
        self.main_tabs.removeTab(0)
               

    def build_gui(self):                

        self.leftWidgetPart.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding))
        self.leftWidgetPart.setMaximumWidth(self.CONTROL_AREA_WIDTH + 20)
        self.leftWidgetPart.updateGeometry()

        box = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-10)

        idx = -1 

        #widget index 2 
        idx += 1 
        box1 = gui.widgetBox(box) 
        self.box_source = gui.comboBox(box1, self, "SOURCE",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['From Oasys wire', 'Normalized to 1 W/eV', 'From external file (eV, W/eV)', 'From external file (eV, phot/s/.1%bw)'],
                    valueType=int, orientation="horizontal", labelWidth=150)
        self.show_at(self.unitFlags()[idx], box1)
        
        #widget index 6 
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "ENER_MIN",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 7 
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "ENER_MAX",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 8 
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "ENER_N",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 9 ***********   File Browser ******************
        idx += 1
        box1 = gui.widgetBox(box)
        file_box_id = oasysgui.widgetBox(box1, "", addSpace=False, orientation="horizontal")
        self.file_id = oasysgui.lineEdit(file_box_id, self, "SOURCE_FILE", self.unitLabels()[idx],
                                    labelWidth=100, valueType=str, orientation="horizontal")
        gui.button(file_box_id, self, "...", callback=self.select_input_file, width=25)
        self.show_at(self.unitFlags()[idx], box1)

        idx += 1 
        box1 = gui.widgetBox(box) 
        self.box_source = gui.comboBox(box1, self, "BM05_CONFIGURATION",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['(0) BM05 ESRF EH2 (0.85T)',
                          '(1) BM18 central axis (1.56T)', '(2) BM18 lateral axis (0.85T)',
                          '(3) BM05 EH2 central axis (0.85T)', '(4) BM05 EH2 side beam (0.39T)',
                          '(5) BM05 EH1 central axis (0.85T)', '(6) BM05 EH1 side beam (0.39T)'
                           ],
                    valueType=int, orientation="horizontal", labelWidth=150)
        self.show_at(self.unitFlags()[idx], box1)              

        #widget index 12 
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "CHOPPER",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1) 
        
        #widget index 13 
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "EXTRA_AIR_BEFORE_SAMPLE",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 14 
        idx += 1 
        box1 = gui.widgetBox(box) 
        self.box_source = gui.comboBox(box1, self, "CALCULATION_TYPE",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['Spectrum at sample surface',
                           'Spectrum after sample',                          
                           'Spectrum at detector level (without sample)',
                           'Spectrum at detector level with sample',
                           'Detected spectrum (without sample)',
                           'Detected spectrum with sample'
                          ],
                    valueType=int, orientation="horizontal", labelWidth=150)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 15
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "PROPAGATION_DISTANCE",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 15
        idx += 1 
        box1 = gui.widgetBox(box) 
        oasysgui.lineEdit(box1, self, "SCINTILLATOR_EFFICIENCY",
                     label=self.unitLabels()[idx], addSpace=False,
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

    def select_input_file(self):
        self.file_id.setText(oasysgui.selectFileFromDialog(self, self.SOURCE_FILE,
                                    "Open 2-columns file with spectral power",
                                    file_extension_filter="ascii dat (*.dat *.txt *spec)"))         
                

    def set_EL_FLAG(self):
        self.initializeTabs()
    

    def unitLabels(self):
         return ['Input beam:',
                 'From energy [eV]:      ',
                 'To energy [eV]:',
                 'Energy points:  ',
                 'File with input beam spectral power:',
                 'BM05 Configuration: ',                 
                 'Chopper (%):',
                 'Extra air before sample (m):',
                 'Spectral calculation',
                 'Propagation distance (m): ',
                 'Scintillator efficiency (ph/keV):'
                ]


    def unitFlags(self):
         return ['True',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  ==  1',
                 'self.SOURCE  >  1',
                 'True',                 
                 'True',
                 'True',
                 'True',
                 'self.CALCULATION_TYPE >= 2',
                 'self.CALCULATION_TYPE >= 4'
                ]

    def get_help_name(self):
        return 'BM05_Calculator'

    def acceptExchangeData(self, exchangeData):

        self.input_spectrum = None
        self.input_script = None
        self.SOURCE = 0        

        try:
            if not exchangeData is None:
                if exchangeData.get_program_name() == "XOPPY":
                    no_bandwidth = False
                    if exchangeData.get_widget_name() =="UNDULATOR_FLUX" :
                        # self.SOURCE_FILE = "xoppy_undulator_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() == "BM" :
                        if exchangeData.get_content("is_log_plot") == 1:
                            raise Exception("Logaritmic X scale of Xoppy Energy distribution not supported")
                        if exchangeData.get_content("calculation_type") == 0 and exchangeData.get_content("psi") in [0,2]:
                            # self.SOURCE_FILE = "xoppy_bm_flux"
                            no_bandwidth = True
                            index_flux = 6
                        else:
                            raise Exception("Xoppy result is not a Flux vs Energy distribution integrated in Psi")
                    elif exchangeData.get_widget_name() =="XWIGGLER" :
                        # self.SOURCE_FILE = "xoppy_xwiggler_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() =="WS" :
                        # self.SOURCE_FILE = "xoppy_xwiggler_flux"
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() =="XTUBES" :
                        # self.SOURCE_FILE = "xoppy_xtubes_flux"
                        index_flux = 1
                        no_bandwidth = True
                    elif exchangeData.get_widget_name() =="XTUBE_W" :
                        # self.SOURCE_FILE = "xoppy_xtube_w_flux"
                        index_flux = 1
                        no_bandwidth = True
                    elif exchangeData.get_widget_name() =="BLACK_BODY" :
                        # self.SOURCE_FILE = "xoppy_black_body_flux"
                        no_bandwidth = True
                        index_flux = 2

                    elif exchangeData.get_widget_name() =="UNDULATOR_RADIATION" :
                        # self.SOURCE_FILE = "xoppy_undulator_radiation"
                        no_bandwidth = True
                        index_flux = 1
                    elif exchangeData.get_widget_name() =="POWER" :
                        # self.SOURCE_FILE = "xoppy_undulator_power"
                        no_bandwidth = True
                        index_flux = -1
                    elif exchangeData.get_widget_name() =="POWER3D" :
                        # self.SOURCE_FILE = "xoppy_power3d"
                        no_bandwidth = True
                        index_flux = 1

                    else:
                        raise Exception("Xoppy Source not recognized")

                    # self.SOURCE_FILE += "_" + str(id(self)) + ".dat"


                    spectrum = exchangeData.get_content("xoppy_data")

                    if exchangeData.get_widget_name() =="UNDULATOR_RADIATION" or \
                        exchangeData.get_widget_name() =="POWER3D":
                        [p, e, h, v ] = spectrum
                        tmp = p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*codata.e*1e3
                        spectrum = numpy.vstack((e,p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*
                                                 codata.e*1e3))
                        self.input_spectrum = spectrum                        
                    else:

                        if not no_bandwidth:
                            spectrum[:,index_flux] /= 0.001*spectrum[:,0]

                        self.input_spectrum = numpy.vstack((spectrum[:,0],spectrum[:,index_flux]))

                    try:
                        self.input_script = exchangeData.get_content("xoppy_script")
                    except:
                        self.input_script = None

                    self.source_of_previous_element = spectrum[:,1]
                    self.process_showers()
                    self.compute()

        except Exception as exception:
            QMessageBox.critical(self, "Error", str(exception), QMessageBox.Ok)

    def check_fields(self):

        self.CHOPPER == congruence.checkPositiveNumber(self.CHOPPER, "Chopper")
        self.EXTRA_AIR_BEFORE_SAMPLE == congruence.checkPositiveNumber(self.EXTRA_AIR_BEFORE_SAMPLE, "Extra air before sample")
                        
    def get_values_from_bm05_config(self):

        """ This function is to get the values of sample position, beam_v_size
        and beam_h_size for a given configuration these values were taken from
        the EXCEL file dose_estimator_BM05-BM18_2023.xlsx
        Sample position is in meters and beam sizes in milimeters """

        if self.BM05_CONFIGURATION == 0:
            config_name = '(0) BM05 ESRF EH2 (0.85T)'
            sample_position = 58.0 
            beam_v_size = 5.0 
            beam_h_size = 10.0

        elif self.BM05_CONFIGURATION == 1:
            config_name = '(1) BM18 central axis (1.56T)'
            sample_position = 175 
            beam_v_size = 10.0 
            beam_h_size = 10.0

        elif self.BM05_CONFIGURATION == 2:
            config_name = '(2) BM18 lateral axis  (0.85T)'
            sample_position = 175 
            beam_v_size = 10.0 
            beam_h_size = 10.0

        elif self.BM05_CONFIGURATION == 3:
            config_name = '(3) BM05 EH2 central axis (0.85T)'
            sample_position = 56.5 # meters
            beam_v_size = 5.0 # mm
            beam_h_size = 10.0 #mm

        elif self.BM05_CONFIGURATION == 4:
            config_name = '(4) BM05 EH2 side beam (0.39T)'                      
            sample_position = 56.5 # meters
            beam_v_size = 5.0 # mm
            beam_h_size = 10.0 #mm

        elif self.BM05_CONFIGURATION == 5:
            config_name = '(5) BM05 EH1 central axis (0.85T)'
            sample_position = 44.0 # meters
            beam_v_size = 5.0 # mm
            beam_h_size = 10.0 #mm
        
        elif self.BM05_CONFIGURATION == 6:
            config_name = '(6) BM05 EH1  side beam (0.39T)'
            sample_position = 44.0 # meters
            beam_v_size = 5.0 # mm
            beam_h_size = 10.0 #mm
        
        else:
            raise RuntimeError("ERROR: Sorry this configuration has not been yet implemented")

        return config_name, sample_position, beam_v_size, beam_h_size
    
    def get_label_calculation_type(self):

        if self.CALCULATION_TYPE == 0:
            cal_type_str = "Spectrum at sample surface"

        elif self.CALCULATION_TYPE == 1:
            cal_type_str = 'Spectrum after sample'

        elif self.CALCULATION_TYPE == 2:
            cal_type_str = 'Spectrum at detector level (without sample)'
        
        elif self.CALCULATION_TYPE == 3:                        
            cal_type_str = 'Spectrum at detector level with sample'
        
        elif self.CALCULATION_TYPE == 4:
            cal_type_str = 'Detected spectrum (without sample)'
        
        elif self.CALCULATION_TYPE == 5:
            cal_type_str = 'Detected spectrum with sample'
        else:
            raise RuntimeError("ERROR: Sorry this calculation type has not been yet implemented")

        return cal_type_str

    def do_xoppy_calculation(self):
    
        if self.SOURCE == 0:
            if self.input_spectrum is None:
                raise Exception("No input beam")
            else:
                energies = self.input_spectrum[0,:].copy()
                source = self.input_spectrum[1,:].copy()
                #if self.CALCULATION_TYPE:
                #    absorbed = source - self.input_spectrum[-1,:].copy()
            if self.input_script is None:
                script_previous = '#\n# >> MISSING SCRIPT TO CREATE (energy, spectral_power) <<\n#\n'
            else:
                script_previous = self.input_script
        elif self.SOURCE == 1:
            energies = numpy.linspace(self.ENER_MIN,self.ENER_MAX,self.ENER_N)
            source = numpy.ones(energies.size)
            tmp = numpy.vstack( (energies,source))
            self.input_spectrum = source
            script_previous = "import numpy\nenergy = numpy.linspace(%g,%g,%d)\nspectral_power = numpy.ones(%d)\n" % \
                        (self.ENER_MIN,self.ENER_MAX,self.ENER_N,self.ENER_N)
        elif self.SOURCE == 2:  # file contains energy_eV and spectral power (W/eV)
            source_file = self.SOURCE_FILE
            try:
                tmp = numpy.loadtxt(source_file)
                energies = tmp[:,0]
                source = tmp[:,1]
                self.input_spectrum = source
                script_previous = "import numpy\ntmp = numpy.loadtxt(%s)\nenergy = tmp[:,0]\nspectral_power = tmp[:,1]\n" % \
                                (source_file)
            except:
                print("Error loading file %s "%(source_file))
                raise
        elif self.SOURCE == 4:  # file contains energy_eV and flux (ph/s/0.1%bw
            source_file = self.SOURCE_FILE
            try:
                tmp = numpy.loadtxt(source_file)
                energies = tmp[:,0]
                source = tmp[:,1] * (codata.e * 1e3)
                self.input_spectrum = source
                script_previous = "import numpy\nimport scipy.constants as codata\ntmp = numpy.loadtxt(%s)\nenergy = tmp[:,0]\nspectral_power = tmp[:,1] / (codata.e * 1e3)\n" % \
                                (source_file)
            except:
                print("Error loading file %s "%(source_file))
                raise
        
        # get values from configuration        
        config_name, sample_position, beam_v_size, beam_h_size = self.get_values_from_bm05_config()        

        chopper = self.CHOPPER  # chpper in percentage
        extra_distance = self.EXTRA_AIR_BEFORE_SAMPLE # distance to project the slit
        energy_bin_size = energies[1] - energies[0]

        # we get the flux for typical cases and for detected at the scintillators
        if self.CALCULATION_TYPE < 4:
            flux = source/codata.e/1e3
        else:
            flux = (self.source_of_previous_element - source)/codata.e/1e3 * energies/1000 * self.SCINTILLATOR_EFFICIENCY 

        # calculate the energy scaling factor
        e_scaling_fac = energy_bin_size / (energies / 1e3)

        # considering the propagation distance
        if self.CALCULATION_TYPE >= 2:            
            extra_distance = self.PROPAGATION_DISTANCE  

        calculation_distance = sample_position + extra_distance 
                    
        #flux at the calculation level per mm2
        flux_calc_level = flux * e_scaling_fac / (beam_v_size * beam_h_size) * \
                (chopper/100) * sample_position**2 / (calculation_distance)**2
        
        #Integrated flux per mm2:
        calculation_type = self.get_label_calculation_type()
        total_flux = sum(flux_calc_level)
        pic_energy = energies[numpy.argmax(flux_calc_level)] / 1e3
        integrated_energy = sum(energies * flux_calc_level / sum(flux_calc_level)) / 1e3
                
        print("Calculation for the configuration %s" % (config_name))
        print("Source has been calculated over a slit at %.1f m" % (sample_position))
        print("Slit horizontal aperture of %.1f mm" % (beam_h_size))
        print("Slit vertical aperture of %.1f mm" % (beam_v_size))

        print("________________BM05_Calculator________________")                
        print(calculation_type)
        print("For a distance of %.3f m" % (calculation_distance))
        print("Flux (ph/s/mm2) %.2e" % (total_flux))
        print("Pic energy %.1f keV" % (pic_energy))
        print("Integrated energy %.1f keV" % (integrated_energy))  
        
        # This is for the script   

        dict_parameters = {           
            "config_name"            : config_name,            
            "calculation_type"       : calculation_type,
            "calculation_distance"   : "%s" % (calculation_distance),           
            "total_flux"             : "{:0.2e}".format(total_flux),       
            "pic_energy"             : "%s" % (round(pic_energy, 1)),       
            "integrated_energy"      : "%s" % (round(integrated_energy, 1))                                 
        }          

        out_dictionary = {}

        #This widget does not calculate any spectrum, so just pass the incoming spectrum
        out_dictionary['data'] = [energies, source]                                         

        script_element = self.script_template().format_map(dict_parameters)

        script = script_previous + script_element        

        self.xoppy_script.set_code(script)

        return out_dictionary, script

    def script_template(self):
        return """
#
# script to make the calculations (created by XOPPY:BM05_Calculator)
#
print("________________BM05_Calculator________________") 
print("Calculation for the configuration {config_name}")
print("{calculation_type}")
print("For a distance of {calculation_distance} m")
print("Flux {total_flux} (ph/s/mm2)")
print("Pic energy {pic_energy} keV")
print("Integrated energy {integrated_energy} keV")

#
# end script
#
"""  
    def extract_data_from_xoppy_output(self, calculation_output):

        """ Calculated data """        

        out_dictionary, script = calculation_output

        cumulated_data = {}                
        results = []

        # First, we add the typical: energy axis and input beam        
        results.append((out_dictionary['data'][0]).tolist()) #Pos: 0
        results.append((out_dictionary['data'][1]).tolist()) #Pos: 1

        cumulated_data['data']=numpy.array(results)
                
        calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
       
        try:
            calculated_data.add_content("xoppy_data", cumulated_data["data"].T)
        except:
            pass
        try:
            calculated_data.add_content("xoppy_script", script)
        except:
            pass
        try:
            calculated_data.add_content("labels", out_dictionary["labels"])
        except:
            pass
        try:
           calculated_data.add_content("info", out_dictionary["info"])
        except:
           pass   
        return calculated_data


    def get_data_exchange_widget_name(self):
        return "POWER"    


if __name__ == "__main__":

    import sys
    input_type = 0

    if input_type == 1:
        from oasys.widgets.exchange import DataExchangeObject
        input_data_type = "POWER"

        if input_data_type == "POWER":
            # create fake UNDULATOR_FLUX xoppy exchange data
            e = numpy.linspace(1000.0, 10000.0, 100)
            source = e/10
            received_data = DataExchangeObject("XOPPY", "POWER")
            received_data.add_content("xoppy_data", numpy.vstack((e,e,source)).T)
            received_data.add_content("xoppy_code", "US")

        elif input_data_type == "POWER3D":
            # create unulator_radiation xoppy exchange data
            from xoppylib.sources.xoppy_undulators import xoppy_calc_undulator_radiation

            e, h, v, p, code = xoppy_calc_undulator_radiation(ELECTRONENERGY=6.04,ELECTRONENERGYSPREAD=0.001,ELECTRONCURRENT=0.2,\
                                               ELECTRONBEAMSIZEH=0.000395,ELECTRONBEAMSIZEV=9.9e-06,\
                                               ELECTRONBEAMDIVERGENCEH=1.05e-05,ELECTRONBEAMDIVERGENCEV=3.9e-06,\
                                               PERIODID=0.018,NPERIODS=222,KV=1.68,DISTANCE=30.0,
                                               SETRESONANCE=0,HARMONICNUMBER=1,
                                               GAPH=0.001,GAPV=0.001,\
                                               HSLITPOINTS=41,VSLITPOINTS=41,METHOD=0,
                                               PHOTONENERGYMIN=7000,PHOTONENERGYMAX=8100,PHOTONENERGYPOINTS=20,
                                               USEEMITTANCES=1)
            received_data = DataExchangeObject("XOPPY", "POWER3D")
            received_data = DataExchangeObject("XOPPY", "UNDULATOR_RADIATION")
            received_data.add_content("xoppy_data", [p, e, h, v])
            received_data.add_content("xoppy_code", code)

        app = QApplication(sys.argv)
        w = OWBM05Calculator()
        w.acceptExchangeData(received_data)
        w.show()
        app.exec()
        w.saveSettings()

    else:
        app = QApplication(sys.argv)
        w = OWBM05Calculator()
        w.SOURCE = 1
        w.show()
        app.exec()
        w.saveSettings()



#
# FilterBlock contains a list of Filters
# FilterBox contains a list of FilterBlock
#

from orangecontrib.esrf.syned.util.syned_filter_with_density import FilterWithDensity

from syned.syned_object import SynedObject
from collections import OrderedDict

class FilterBlock(SynedObject):
    def __init__(self, filters_list=None):
        if filters_list is None:
            self._filters_list = []
        else:
            self._filters_list = filters_list

        # support text containg name of variable, help text and unit. Will be stored in self._support_dictionary
        self._set_support_text([
                    ("filters_list",  "Filters list", ""),
            ] )


    # # overwrites the SynedObject method for dealing with list
    def to_dictionary(self):
        """
        Returns a dictionary with the object fields.

        Returns
        -------
        dict
            A dictionary with the data.

        """
        dict_to_save = OrderedDict()
        dict_to_save.update({"CLASS_NAME":self.__class__.__name__})
        dict_to_save["filters_list"] = [el.to_dictionary() for el in self._filters_list]
        return dict_to_save

    def get_n(self):
        return len(self._filters_list)

    def get_item(self, index):
        return self._filters_list[index]

class FilterBox(SynedObject):
    def __init__(self, filter_blocks_list=None):
        if filter_blocks_list is None:
            self._filter_blocks_list = []
        else:
            self._filter_blocks_list = filter_blocks_list

        self.__selection = [] # to save the status

        # support text containg name of variable, help text and unit. Will be stored in self._support_dictionary
        self._set_support_text([
                    ("filter_blocks_list",  "list of blocks (axes) of filters", ""),
            ] )


    # # overwrites the SynedObject method for dealing with list
    def to_dictionary(self):
        """
        Returns a dictionary with the object fields.

        Returns
        -------
        dict
            A dictionary with the data.

        """
        dict_to_save = OrderedDict()
        dict_to_save.update({"CLASS_NAME":self.__class__.__name__})
        dict_to_save["filter_blocks_list"] = [el.to_dictionary() for el in self._filter_blocks_list]
        return dict_to_save

    def get_n(self):
        return len(self._filter_blocks_list)

    def get_item(self, index):
        return self._filter_blocks_list[index]

    def get_selection(self):
        return self.__selection

    def set_selection(self, selection):
        self.__selection = selection

if __name__ == "__main__":

    # f1 = FilterWithDensity(name='f1', material='Si', thickness=30e-6)
    # f2 = FilterWithDensity(name='f2', material='W', thickness=30e-6)
    # f3 = FilterWithDensity(name='f3', material='K', thickness=30e-6)
    # f4 = FilterWithDensity(name='f4', material='Cu', thickness=30e-6)
    # f5 = FilterWithDensity(name='f5', material='Ag', thickness=30e-6)
    #
    # bf = FilterBlock(filters_list=[f1,f2,f3,f4])
    #
    # # print(bf.info())
    # # print(bf.to_dictionary())
    # # print(bf.to_json())
    #
    # box = FilterBox(filter_blocks_list=[bf, bf])
    #
    # print(box.to_json(file_name="tmp.json"))

    # from syned.util.json_tools import load_from_json_file
    # from orangecontrib.syned.util.filter_block import FilterBlock, FilterBox
    #
    # tmp = load_from_json_file("tmp.json",
    #                           exec_commands=[
    #                               "from orangecontrib.syned.util.filter_with_density import FilterWithDensity",
    #                               "from orangecontrib.syned.util.filter_block import FilterBlock, FilterBox",
    #                           ])
    #
    # print(tmp.info())

    # create json with suned FilterBox

    if 0:
        import os
        import json
        import orangecanvas.resources as resources
        file_json = os.path.join(resources.package_dirname("orangecontrib.esrf.xoppy.data"), 'bm05_wb_attenuators.json')
        with open(file_json) as att_file:
            att_dic = json.load(att_file)

        n_keys = 0
        keys = []
        for key in att_dic.keys():
            n_keys += 1
            keys.append(key)

        # update combo boxes
        block_list = []
        for i in range(n_keys):
            items = []
            for filter in att_dic[keys[i]].keys():
                item = att_dic[keys[i]][filter]
                f = FilterWithDensity(name=item['name'],
                                      material=item['substance'],
                                      thickness=item['thickness'],
                                      density=item['thickness'])
                items.append(f)

            block_list.append(FilterBlock(filters_list=items))

        box = FilterBox(filter_blocks_list=block_list)

        print(box.to_json(file_name='tmp.json'))

    # read the syned json file

    if 1:
        from syned.util.json_tools import load_from_json_file
        tmp = load_from_json_file("tmp.json",
                                  exec_commands=[
                                      "from orangecontrib.syned.util.filter_with_density import FilterWithDensity",
                                      "from orangecontrib.syned.util.filter_block import FilterBlock, FilterBox",
                                  ])

        print(tmp.info())




import os, sys
import json
import urllib.request
from PyQt5.QtWidgets import QMessageBox

# import orangecanvas.resources as resources

from orangewidget import gui
from orangewidget.settings import Setting

from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence

from orangecontrib.syned.widgets.gui.ow_optical_element import OWOpticalElement
from orangecontrib.esrf.syned.util.syned_filter_with_density import FilterWithDensity
from orangecontrib.esrf.syned.util.syned_filter_packs import FilterBox, FilterBlock


from syned.util.json_tools import load_from_json_file, load_from_json_url

class OWBoxOfFilters(OWOpticalElement):

    name = "Box of Filters"
    description = "Syned: Box of Filters"
    icon = "icons/box_of_filters.png"
    priority = 3.1

    att1 = Setting(0)
    att2 = Setting(0)
    att3 = Setting(0)
    att4 = Setting(0)
    att5 = Setting(0)

    n_blocks = Setting(3)
    syned_file_name = Setting("Select *.json file")
    # syned_file_name = Setting("/users/srio/OASYS1.2/OASYS1-ESRF-EXTENSIONS/orangecontrib/esrf/syned/util/tmp.json")
    # syned_file_name = Setting(os.path.join(resources.package_dirname("orangecontrib.esrf.xoppy.data"), 'bm05_wb_attenuators.json'))
    syned_file_name = Setting("https://raw.githubusercontent.com/oasys-esrf-kit/OASYS1-ESRF-Extensions/master/orangecontrib/esrf/xoppy/data/bm05_wb_attenuators.json")

    syned_filterbox = FilterBox()
    # att_dic = Setting(None)

    def __init__(self):
        super().__init__(allow_angle_radial=False, allow_angle_azimuthal=False)

    def draw_specific_box(self):

        #################
        box_json = oasysgui.widgetBox(self.tab_bas, "json files i/o", addSpace=True, orientation="vertical")

        # box_json =  oasysgui.widgetBox(files_box, "Read/Write File", addSpace=False, orientation="vertical")

        file_box = oasysgui.widgetBox(box_json, "", addSpace=False, orientation="horizontal")

        self.le_syned_file_name = oasysgui.lineEdit(file_box, self, "syned_file_name", "File Name/URL",
                                                    labelWidth=150, valueType=str, orientation="horizontal")

        gui.button(file_box, self, "...", callback=self.select_syned_file, width=25)

        button_box = oasysgui.widgetBox(box_json, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Read Syned File", callback=self.read_syned_file)
        button.setFixedHeight(25)

        button = gui.button(button_box, self, "Read plane json File", callback=self.read_plane_json_file)
        button.setFixedHeight(25)

        button = gui.button(button_box, self, "Write Syned File...", callback=self.write_syned_file)
        button.setFixedHeight(25)



        ################
        filter_box = oasysgui.widgetBox(self.tab_bas, "Box of Filters Setting", addSpace=True, orientation="vertical")


        box1 = gui.widgetBox(filter_box)
        gui.comboBox(box1, self, "n_blocks",
                                       label="Number of blocks or axes", addSpace=False,
                                       items=['0','1','2','3','4','5'], callback=self.set_visibility,
                                       valueType=int, orientation="horizontal", labelWidth=250, editable=0)

        self.wid_att1 = gui.widgetBox(box1)
        self.wid_att1_combo = gui.comboBox(self.wid_att1, self, "att1",
                                       label="Att1", addSpace=False,
                                       items=['Empty'],
                                       valueType=int, orientation="horizontal", labelWidth=150, editable=1)
        self.wid_att2 = gui.widgetBox(box1)
        self.wid_att2_combo = gui.comboBox(self.wid_att2, self, "att2",
                                       label="Att2", addSpace=False,
                                       items=['Empty'],
                                       valueType=int, orientation="horizontal", labelWidth=150, editable=1)
        self.wid_att3 = gui.widgetBox(box1)
        self.wid_att3_combo = gui.comboBox(self.wid_att3, self, "att3",
                                       label="Att3", addSpace=False,
                                       items=['Empty'],
                                       valueType=int, orientation="horizontal", labelWidth=150, editable=1)
        self.wid_att4 = gui.widgetBox(box1)
        self.wid_att4_combo = gui.comboBox(self.wid_att4, self, "att4",
                                       label="Att4", addSpace=False,
                                       items=['Empty'],
                                       valueType=int, orientation="horizontal", labelWidth=150, editable=1)
        self.wid_att5 = gui.widgetBox(box1)
        self.wid_att5_combo = gui.comboBox(self.wid_att5, self, "att5",
                                       label="Att5", addSpace=False,
                                       items=['Empty'],
                                       valueType=int, orientation="horizontal", labelWidth=150, editable=1)

        self.set_visibility()

        # if self.att_dic is not None:
        #     self.configure_blocks_from_syned_json(self._att_dic_to_syned_filterbox(self.att_dic))

    def set_visibility(self):
        self.wid_att1.setVisible(False)
        self.wid_att2.setVisible(False)
        self.wid_att3.setVisible(False)
        self.wid_att4.setVisible(False)
        self.wid_att5.setVisible(False)

        if self.n_blocks >= 1: self.wid_att1.setVisible(True)
        if self.n_blocks >= 2: self.wid_att2.setVisible(True)
        if self.n_blocks >= 3: self.wid_att3.setVisible(True)
        if self.n_blocks >= 4: self.wid_att4.setVisible(True)
        if self.n_blocks >= 5: self.wid_att5.setVisible(True)

    def select_syned_file(self):
        self.le_syned_file_name.setText(oasysgui.selectFileFromDialog(self, self.syned_file_name, "Open json File"))

    def read_syned_file(self):
        try:
            congruence.checkEmptyString(self.syned_file_name, "Syned File Name/Url")

            if (len(self.syned_file_name) > 7 and self.syned_file_name[:7] == "http://") or \
                (len(self.syned_file_name) > 8 and self.syned_file_name[:8] == "https://"):
                congruence.checkUrl(self.syned_file_name)
                is_remote = True
            else:
                congruence.checkFile(self.syned_file_name)
                is_remote = False

            try:
                if is_remote:
                    content = load_from_json_url(self.syned_file_name,                                  exec_commands=[
                                      "from orangecontrib.esrf.syned.util.syned_filter_with_density import FilterWithDensity",
                                      "from orangecontrib.esrf.syned.util.syned_filter_packs import FilterBlock, FilterBox"])
                else:
                    content = load_from_json_file(self.syned_file_name,                                  exec_commands=[
                                      "from orangecontrib.esrf.syned.util.syned_filter_with_density import FilterWithDensity",
                                      "from orangecontrib.esrf.syned.util.syned_filter_packs import FilterBlock, FilterBox"])

                if isinstance(content, FilterBox):
                    self.configure_blocks_from_syned_json(content)
                    self.syned_filterbox = content
                else:
                    raise Exception("json file must contain a SYNED FilterBox")
            except Exception as e:
                raise Exception("Error reading SYNED FilterBox from file: " + str(e))
        except Exception as e:
            QMessageBox.critical(self, "Error", str(e.args[0]), QMessageBox.Ok)

    def _att_dic_to_syned_filterbox(self, att_dic):
        n_keys = 0
        keys = []
        for key in att_dic.keys():
            n_keys += 1
            keys.append(key)

        # update combo boxes
        block_list = []
        for i in range(n_keys):
            items = []
            for filter in att_dic[keys[i]].keys():
                if filter[0] != "_":
                    item = att_dic[keys[i]][filter]
                    f = FilterWithDensity(name=item['name'],
                                          material=item['substance'],
                                          thickness=item['thickness'],
                                          density=item['thickness'])
                    items.append(f)

            block_list.append(FilterBlock(filters_list=items))

        return FilterBox(filter_blocks_list=block_list)

    def read_plane_json_file(self):
        try:
            congruence.checkEmptyString(self.syned_file_name, "plane json File Name/Url")

            if (len(self.syned_file_name) > 7 and self.syned_file_name[:7] == "http://") or \
                (len(self.syned_file_name) > 8 and self.syned_file_name[:8] == "https://"):
                congruence.checkUrl(self.syned_file_name)
                is_remote = True
            else:
                congruence.checkFile(self.syned_file_name)
                is_remote = False

            try:
                if is_remote:
                    # raise Exception("URL not implemented for plane json files")
                    response = urllib.request.urlopen(self.syned_file_name)
                    att_dic = json.load(response)
                else:
                    with open(self.syned_file_name) as att_file:
                        att_dic = json.load(att_file)

                if isinstance(att_dic, dict):
                    self.configure_blocks_from_syned_json(self._att_dic_to_syned_filterbox(att_dic))
                    self.syned_filterbox = self._att_dic_to_syned_filterbox(att_dic)
                else:
                    raise Exception("json file must contain a FilterBox")
            except Exception as e:
                raise Exception("Error reading filter box from plane json file: " + str(e))
        except Exception as e:
            QMessageBox.critical(self, "Error", str(e.args[0]), QMessageBox.Ok)

    def write_syned_file(self):
        try:
            filename = oasysgui.selectSaveFileFromDialog(self, message="Save File", default_file_name="", file_extension_filter="*.*")
            if filename is not None:
                self.syned_filterbox.to_json(filename)
                QMessageBox.information(self, "File Save", "JSON file %s correctly written to disk" % filename, QMessageBox.Ok)
        except Exception as e:
            QMessageBox.critical(self, "Error", str(e.args[0]), QMessageBox.Ok)

    def update_combo(self, id, new_input):
        n_old = id.count()
        n = len(new_input)
        for i in range(n):
            if i >= n_old:
                id.addItem(new_input[i])
            else:
                id.setItemText(i, new_input[i])

    def configure_blocks_from_syned_json(self, content):
        if content.get_n() > 5:
            raise Exception('Maximum of 5 blocks allowed.')
        else:
            self.n_blocks = content.get_n()

        self.set_visibility()

        # update combo boxes
        for i in range(content.get_n()):
            blc = content.get_item(i)
            items = []
            for j in range(blc.get_n()):
                filter = blc.get_item(j)
                items.append(filter.get_name())
            if i == 0: self.update_combo(self.wid_att1_combo, items)
            if i == 1: self.update_combo(self.wid_att2_combo, items)
            if i == 2: self.update_combo(self.wid_att3_combo, items)
            if i == 3: self.update_combo(self.wid_att4_combo, items)
            if i == 4: self.update_combo(self.wid_att5_combo, items)

    def get_optical_element(self):
        self.syned_filterbox.set_selection([self.att1, self.att2, self.att3s, self.att4, self.att5])
        return self.syned_filterbox


if __name__ == "__main__":
    from PyQt5.QtWidgets import QApplication
    app = QApplication(sys.argv)
    w = OWBoxOfFilters()
    w.show()
    app.exec()




import os, sys
import numpy

from syned.storage_ring.light_source import LightSource, ElectronBeam
from syned.beamline.beamline import Beamline

from PyQt5.QtGui import QPalette, QColor, QFont
from PyQt5.QtWidgets import QMessageBox
from PyQt5.QtCore import QRect

from orangewidget import gui
from orangewidget.settings import Setting

from oasys.widgets.widget import OWWidget
from oasys.widgets import gui as oasysgui
from oasys.widgets.gui import MessageDialog, ConfirmDialog

import orangecanvas.resources as resources

import at #accelerator toolbox

from oasys.util.oasys_util import EmittingStream

from PyQt5.QtWidgets import QApplication
from PyQt5.QtGui import QTextCursor

VERTICAL = 1
HORIZONTAL = 2
BOTH = 3

lattice_file = os.path.join(resources.package_dirname("orangecontrib.esrf.syned.data"), 'S28F_all_BM.mat')
AT_LATTICE = at.load_lattice(lattice_file)


def get_electron_beam_parameters_from_at(r0=AT_LATTICE, id_number=1, s_locs=None, npoints=10, verbose=False):
    """
    Computes optics geometry and beam sigmas at given s locations

    Parameters
    ----------
    r0:         pyAT Lattice
    id_number:  ID cell section number
    s_locs:     None: covers all cell with npoints; numpy.array: user-defined points
    npoints:    number of points (when s_locs=None)
    verbose:    flag to verbose output

    Returns:
        data, epsilonX, epsilonY, labels, s0
    -------

    """

    id = f'ID{id_number:02d}'
    # print(">>> id", id)
    IDind = r0.get_uint32_index(id)
    s0 = r0.get_s_pos(IDind)[0]

    cell_length = (r0.get_s_pos(at.End) / 32)[0]

    if s_locs is None:
        s_locs = numpy.linspace(s0, s0 + cell_length, npoints)
    else:
        s_locs = numpy.array(s_locs)


    # get circumference
    Circumference = r0.get_s_pos(at.End)

    # define break locations
    npoints = s_locs.shape[0]

    # insert markers at break locations
    r = r0.sbreak(break_s=list(s_locs))

    # indexes of s locations
    s_ind = r.get_uint32_index('sbreak')

    # get lattice parameters with radiation
    if verbose:
        print('get lattice parameters')
    r.enable_6d()
    p0 = r.envelope_parameters()

    epsilonX = p0.emittances[0];
    epsilonY = 10 * 1e-12;  # tuned to this value during operation
    delta = p0.sigma_e;

    # gert optics
    if verbose:
        print('get orbit, dispersion, beta functions')
    _, _, l = r.linopt6(refpts=s_ind)

    # get geometry
    if verbose:
        print('get geomtery')
    geom, _ = r.get_geometry(refpts=s_ind)

    data = []

    def _get_cell_for_s(s):
        s1 = s - s0

        alpha = l[i].alpha
        alphaX = alpha[0]
        alphaY = alpha[1]

        beta = l[i].beta
        betaX = beta[0]
        betaY = beta[1]

        gammaX = (1.0 + alpha[0] * alpha[0]) / beta[0]
        gammaY = (1.0 + alpha[1] * alpha[1]) / beta[1]

        eta = l[i].dispersion
        etaX = eta[0]
        etaXp = eta[1]
        etaY = eta[2]
        etaYp = eta[3]

        xx = betaX * epsilonX + (etaX * delta) ** 2
        yy = betaY * epsilonY + (etaY * delta) ** 2
        xxp = -alphaX * epsilonX + etaX * etaXp * delta ** 2
        yyp = -alphaY * epsilonY + etaY * etaYp * delta ** 2
        xpxp = gammaX * epsilonX + (etaXp * delta) ** 2
        ypyp = gammaY * epsilonY + (etaYp * delta) ** 2

        lab_x = geom[i].x
        lab_y = geom[i].y
        angle = geom[i].angle

        return [s1,     # 0
               s,       # 1
               lab_x,   # 2
               lab_y,   # 3
               angle,   # 4
               alphaX,  # 5
               alphaY,  # 6
               betaX,   # 7
               betaY,   # 8
               gammaX, gammaY,                # 9,10
               etaX, etaY, etaXp, etaYp,      # 11-14
               xx, yy, xxp, yyp, xpxp, ypyp,  # 15-20
               numpy.sqrt(xx), numpy.sqrt(yy), numpy.sqrt(xpxp),                # 21-23
               numpy.sqrt(ypyp), numpy.sqrt(xx * xpxp), numpy.sqrt(yy * ypyp),  # 24-26
               ]

    for i in range(0, npoints):
        tmp = _get_cell_for_s(s_locs[i])
        data.append(tmp)


    labels = ['s0',  # 0
           's',  # 1
           'lab_x',  # 2
           'lab_y',  # 3
           'angle',  # 4
           'alphaX',  # 5
           'alphaY',  # 6
           'betaX',  # 7
           'betaY',  # 8
           'gammaX',
           'gammaY',  # 9,10
           'etaX',
           'etaY',
           'etaXp',
           'etaYp',  # 11-14
           'xx',
           'yy',
           'xxp',
           'yyp',
           'xpxp',
           'ypyp',  # 15-20
           'sigma x', # '1e6 * numpy.sqrt(xx),'
           'sigma y', # '1e6 * numpy.sqrt(yy),'
           "sigma x'", # '1e6 * numpy.sqrt(xpxp),  # 21-23
           "sigma y'", # '1e6 * numpy.sqrt(ypyp),'
           "<x * x'>",
           "<y * y'>",  # 24-26
    ]
    return numpy.array(data), epsilonX, epsilonY, labels, s0

class OWEBSCELL(OWWidget):

    name = "ESRF-EBS Cell Parameters"
    description = "Syned: ESRF-EBS Cell Parameters"
    icon = "icons/at_ebs.png"
    priority = 1.2


    maintainer = "Manuel Sanchez del Rio"
    maintainer_email = "srio(@at@)esrf.eu"
    category = "ESRF-EBS Syned Tools"
    keywords = ["data", "file", "load", "read"]

    outputs = [{"name":"SynedData",
                "type":Beamline,
                "doc":"Syned Beamline",
                "id":"data"}]


    want_main_area = 1

    MAX_WIDTH = 1320
    MAX_HEIGHT = 700

    IMAGE_WIDTH = 860
    IMAGE_HEIGHT = 645

    CONTROL_AREA_WIDTH = 405
    TABS_AREA_HEIGHT = 650

    TABS_AREA_HEIGHT = 625
    CONTROL_AREA_WIDTH = 450


    type_of_properties = Setting(0)
    number_of_points  = Setting(250)
    ebs_id_index = Setting(0)
    shift_from_center_of_straight_section = Setting(0.0)

    data = None
    values_at_undulator_center = None

    def __init__(self):

        button_box = oasysgui.widgetBox(self.controlArea, "", addSpace=False, orientation="horizontal")

        button = gui.button(button_box, self, "Calculate/Refresh", callback=self.refresh)
        font = QFont(button.font())
        font.setBold(True)
        button.setFont(font)
        palette = QPalette(button.palette()) # make a copy of the palette
        palette.setColor(QPalette.ButtonText, QColor('Dark Blue'))
        button.setPalette(palette) # assign new palette
        button.setFixedHeight(45)

        gui.separator(self.controlArea)

        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width() * 0.05),
                               round(geom.height() * 0.05),
                               round(min(geom.width() * 0.98, self.MAX_WIDTH)),
                               round(min(geom.height() * 0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())

        self.controlArea.setFixedWidth(self.CONTROL_AREA_WIDTH)

        self.tabs_setting = oasysgui.tabWidget(self.controlArea)
        self.tabs_setting.setFixedHeight(self.TABS_AREA_HEIGHT)
        self.tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)

        self.tab_sou = oasysgui.createTabPage(self.tabs_setting, "EBS Cell Setting")


        self.electron_beam_box = oasysgui.widgetBox(self.tab_sou, "Storage Ring Parameters", addSpace=False, orientation="vertical")

        gui.comboBox(self.electron_beam_box, self, "type_of_properties", label="accelerator toolbox file", labelWidth=250,
                     items=["EBS (S28F 140pm H, 10pm V)"], callback=self.update_cell_file,
                     sendSelectedValue=False, orientation="horizontal")

        gui.comboBox(self.electron_beam_box, self, "ebs_id_index", label="Load parameters from ID cell: ", labelWidth=350,
                     items=self.get_id_list(), callback=self.set_id, sendSelectedValue=False, orientation="horizontal")

        oasysgui.lineEdit(self.electron_beam_box, self, "number_of_points", "number of points in a cell (for plots)",
                          labelWidth=360, valueType=int,
                          orientation="horizontal", callback=self.refresh)

        oasysgui.lineEdit(self.electron_beam_box, self, "shift_from_center_of_straight_section", "undulator center from straight section center",
                          labelWidth=360, valueType=float,
                          orientation="horizontal", callback=self.refresh)

        gui.rubber(self.controlArea)

        self.initializeTabs()


    def get_id_list(self):
        out_list = [("ID%02d" % i)  for i in range(1, 33)]
        return out_list

    def titles(self):
        return ["Beta", "Alpha", "Dispersion", "sigma", "sigma'"]

    def xtitles(self):
        return ['s [m]'] * len(self.titles())

    def ytitles(self):
        return ['beta [m]', 'alpha', 'eta', 'sigma [m]', 'sigma prime [rad]']

    def initializeTabs(self):
        self.tabs = oasysgui.tabWidget(self.mainArea)
        self.tab = []

        n_plots = len(self.titles())

        for i in range(len(self.titles())):
            self.tab.append(oasysgui.createTabPage(self.tabs, self.titles()[i]))

        self.tab.append(oasysgui.createTabPage(self.tabs, "Info"))

        for tab in self.tab:
            tab.setFixedHeight(self.IMAGE_HEIGHT)
            tab.setFixedWidth(self.IMAGE_WIDTH)

        self.info_id = oasysgui.textArea(height=self.IMAGE_HEIGHT-5, width=self.IMAGE_WIDTH-5)
        profile_box = oasysgui.widgetBox(self.tab[-1], "", addSpace=True, orientation="horizontal",
                                         height = self.IMAGE_HEIGHT, width=self.IMAGE_WIDTH-5)
        profile_box.layout().addWidget(self.info_id)

        self.plot_canvas = [None] * n_plots

        for i in range(n_plots):
            self.plot_canvas[i] = oasysgui.plotWindow(roi=False, control=False, position=True)
            self.plot_canvas[i].setDefaultPlotLines(True)
            self.plot_canvas[i].setActiveCurveColor(color='blue')
            self.plot_canvas[i].setGraphXLabel(self.xtitles()[i])
            self.plot_canvas[i].setGraphYLabel(self.ytitles()[i])
            self.plot_canvas[i].setGraphTitle(self.titles()[i])
            self.plot_canvas[i].setInteractiveMode(mode='zoom')


        for index in range(n_plots):
            self.tab[index].layout().addWidget(self.plot_canvas[index])

        self.tabs.setCurrentIndex(n_plots)

    def get_id_number(self):
        return self.ebs_id_index + 1

    def update_cell_file(self):
        pass

    def set_id(self):
        pass

    def plot_graph(self, plot_canvas_index, curve_name, x_values, y_values, xtitle="", ytitle="",
                   color='blue', replace=True):
        self.plot_canvas[plot_canvas_index].addCurve(x_values, y_values, curve_name, symbol='', color=color, replace=replace) #'+', '^', ','
        self.plot_canvas[plot_canvas_index].setGraphXLabel(xtitle)
        self.plot_canvas[plot_canvas_index].setGraphYLabel(ytitle)
        self.plot_canvas[plot_canvas_index].replot()

    def update_plots(self):
        # ALL
        INDICES = [[7, 8], [5, 6], [11, 12], [21, 22], [23, 24]]
        colors = ['black', 'red']

        for iplot in range(5):
            indices = INDICES[iplot]

            xtitle = self.xtitles()[iplot] # "s [m]"
            ytitle = self.ytitles()[iplot] # "Alpha [m]"
            title = self.titles()[iplot]   #
            labels = ["%s X" % title, "%s Y" % title]

            for i, index in enumerate(indices):
                self.plot_canvas[iplot].addCurve(self.data[:, 0], self.data[:, index],
                                             labels[i],
                                             xlabel=xtitle, ylabel=ytitle,
                                             symbol='', color=colors[i])


            self.plot_canvas[iplot].getLegendsDockWidget().setFixedHeight(150)
            self.plot_canvas[iplot].getLegendsDockWidget().setVisible(True)
            self.plot_canvas[iplot].setActiveCurve(labels[0])
            self.plot_canvas[iplot].replot()



    def writeStdOut(self, text):
        cursor = self.info_id.textCursor()
        cursor.movePosition(QTextCursor.End)
        cursor.insertText(text)
        self.info_id.setTextCursor(cursor)
        self.info_id.ensureCursorVisible()

    def refresh(self):
        n_id = self.get_id_number()

        try:
            self.data, epsilonX, epsilonY, labels, s0 = \
                get_electron_beam_parameters_from_at(id_number=n_id,
                                                    npoints=self.number_of_points,
                                                    verbose=1)
        except Exception as exception:
            MessageDialog.message(self, str(exception), "Exception occured in OASYS", "critical")
            if self.IS_DEVELOP: raise exception
            return

        self.info_id.setText("")

        sys.stdout = EmittingStream(textWritten=self.writeStdOut)

        print("\n\n")

        print("Values calculated using the Accelerator Toolbox https://github.com/atcollab/at")
        print("with file %s\n" % lattice_file)

        print("\n============== data for cell sector ID%02d ================\n" % n_id)
        print("epsilon_x = %.3g m.rad" % (epsilonX))
        print("epsilon_y = %.3g m.rad\n\n" % (epsilonY))


        print("============== data at center of the straight section (s=%f m)" % s0)
        for i in range(len(labels)):
            print("%s = %.5g " % (labels[i], self.data[0,i]))

        # data at undulator center
        self.values_at_undulator_center, epsilonX, epsilonY, labels, s0 = \
            get_electron_beam_parameters_from_at(id_number=n_id,
                                                 s_locs=[s0 + self.shift_from_center_of_straight_section],
                                                 npoints=1,
                                                 verbose=1)

        print("\n\n")
        print("============== data at undulator center at %f m from center of the straight section\n" % self.shift_from_center_of_straight_section)
        for i in range(len(labels)):
            print("%s = %.5g " % (labels[i], self.values_at_undulator_center[0,i]))

        self.update_plots()

        # senf syned data
        try:
            self.send("SynedData",
                      Beamline(light_source=LightSource(name="EBS data",
                        electron_beam=self.get_electron_beam(),
                        magnetic_structure=None)))
        except Exception as e:
            QMessageBox.critical(self, "Error", str(e.args[0]), QMessageBox.Ok)
            self.setStatusMessage("")


    def get_electron_beam(self):
        return ElectronBeam(energy_in_GeV=6.0,
                                     energy_spread=0.001,
                                     current=0.2,
                                     number_of_bunches=0,
                                     moment_xx   =self.values_at_undulator_center[0,15],
                                     moment_xxp  =self.values_at_undulator_center[0,17],
                                     moment_xpxp =self.values_at_undulator_center[0,19],
                                     moment_yy   =self.values_at_undulator_center[0,16],
                                     moment_yyp  =self.values_at_undulator_center[0,18],
                                     moment_ypyp =self.values_at_undulator_center[0,20],
                                     )

    def callResetSettings(self):
        if ConfirmDialog.confirmed(parent=self, message="Confirm Reset of the Fields?"):
            try:
                self.resetSettings()
            except:
                pass


if __name__ == "__main__":
    a = QApplication(sys.argv)
    ow = OWEBSCELL()
    ow.show()
    a.exec_()


    # data, epsilonX, epsilonY, labels = get_electron_beam_parameters_from_at(id_number=1, npoints=250)
    # from srxraylib.plot.gol import plot
    # plot(data[:, 0], data[:, 7],
    #      data[:, 0], data[:, 8])





import sys
import numpy
from PyQt5.QtWidgets import QApplication, QMessageBox, QSizePolicy

from orangewidget import gui
from orangewidget.settings import Setting
from oasys.widgets import gui as oasysgui, congruence
from oasys.widgets.exchange import DataExchangeObject
from orangecontrib.xoppy.widgets.gui.ow_xoppy_widget import XoppyWidget

from xoppylib.power.xoppy_calc_power_monochromator import xoppy_calc_power_monochromator

import scipy.constants as codata

class Monochromator(XoppyWidget):
    name = "Monochromator"
    id = "orange.widgets.dataxpower"
    description = "Power Absorbed and Transmitted by Optical Elements"
    icon = "icons/id19_monochromator.png"
    priority = 3
    category = ""
    keywords = ["xoppy", "power", "monochromator"]

    inputs = [("ExchangeData", DataExchangeObject, "acceptExchangeData")]

    SOURCE = Setting(2)
    TYPE = Setting(3)
    N_REFLECTIONS = Setting(2)
    POLARIZATION = Setting(0)
    ENER_SELECTED = Setting(8000)
    H_MILLER = Setting (1)
    K_MILLER = Setting (1)
    L_MILLER = Setting (1)
    THICK = Setting(15)
    ENER_MIN = Setting(7990)
    ENER_MAX = Setting(8010)
    ENER_N = Setting(2000)
    SOURCE_FILE = Setting("?")
    FILE_DUMP = Setting(0)
    ML_H5_FILE = Setting("<none>")
    ML_GRAZING_ANGLE_DEG = Setting(0.2)
    METHOD = Setting(0) # Zachariasen

    input_spectrum = None
    input_script = None

    def __init__(self):
        super().__init__(show_script_tab=True)

    def build_gui(self):

        self.leftWidgetPart.setSizePolicy(QSizePolicy(QSizePolicy.MinimumExpanding, QSizePolicy.MinimumExpanding))
        self.leftWidgetPart.setMaximumWidth(self.CONTROL_AREA_WIDTH + 20)
        self.leftWidgetPart.updateGeometry()

        box_main = oasysgui.widgetBox(self.controlArea, self.name + " Input Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-10)

        idx = -1

        box = oasysgui.widgetBox(box_main, "Input Beam Parameters", orientation="vertical", width=self.CONTROL_AREA_WIDTH-10)
        # widget index 1
        idx += 1
        box1 = gui.widgetBox(box)
        self.box_source = gui.comboBox(box1, self, "SOURCE",
                                       label=self.unitLabels()[idx], addSpace=False,
                                       items=['From Oasys wire', 'Normalized to 1',
                                              'From external file.                '],
                                       valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)


        # widget index 2
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "ENER_MIN",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 3
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "ENER_MAX",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 4
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "ENER_N",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 5 ***********   File Browser ******************
        idx += 1
        box1 = gui.widgetBox(box)
        file_box_id = oasysgui.widgetBox(box1, "", addSpace=False, orientation="horizontal")
        self.file_id = oasysgui.lineEdit(file_box_id, self, "SOURCE_FILE", self.unitLabels()[idx],
                                         labelWidth=100, valueType=str, orientation="horizontal")
        gui.button(file_box_id, self, "...", callback=self.select_input_file, width=25)
        self.show_at(self.unitFlags()[idx], box1)

        box = oasysgui.widgetBox(box_main, "Monochromator", orientation="vertical", width=self.CONTROL_AREA_WIDTH-10)
        # widget index 6
        idx += 1
        box1 = gui.widgetBox(box)
        self.box_source = gui.comboBox(box1, self, "TYPE",
                                       label=self.unitLabels()[idx], addSpace=False,
                                       items=['Empty','Si Bragg','Si Laue',
                                              'Multilayer'],
                                       valueType=int, orientation="horizontal", labelWidth=200)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 7
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "N_REFLECTIONS",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 8
        idx += 1
        box1 = gui.widgetBox(box)
        self.box_source = gui.comboBox(box1, self, "POLARIZATION",
                                       label=self.unitLabels()[idx], addSpace=False,
                                       items=['Sigma','Pi','Unpolarized'],
                                       valueType=int, orientation="horizontal", labelWidth=200)
        self.show_at(self.unitFlags()[idx], box1)


        # widget index 9
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "ENER_SELECTED",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 10
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "H_MILLER",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 11
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "K_MILLER",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 12
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "L_MILLER",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 13
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "THICK",
                                       label=self.unitLabels()[idx], addSpace=False,
                                       valueType=float, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 14
        idx += 1
        box1 = gui.widgetBox(box)
        gui.comboBox(box1, self, "METHOD",
                     label=self.unitLabels()[idx], addSpace=True,
                     items=["Zachariasen", "Guigay"],
                     orientation="horizontal")
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 15
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "ML_H5_FILE",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=str, orientation="horizontal", labelWidth=200)
        self.show_at(self.unitFlags()[idx], box1)

        # widget index 16
        idx += 1
        box1 = gui.widgetBox(box)
        oasysgui.lineEdit(box1, self, "ML_GRAZING_ANGLE_DEG",
                          label=self.unitLabels()[idx], addSpace=False,
                          valueType=float, orientation="horizontal", labelWidth=200)
        self.show_at(self.unitFlags()[idx], box1)

        #widget index 17
        idx += 1
        box1 = gui.widgetBox(box)
        gui.separator(box1, height=7)

        gui.comboBox(box1, self, "FILE_DUMP",
                     label=self.unitLabels()[idx], addSpace=False,
                    items=['No', 'Yes (monochromator.spec)'],
                    valueType=int, orientation="horizontal", labelWidth=250)
        self.show_at(self.unitFlags()[idx], box1)

        self.input_spectrum = None

    def select_input_file(self):
        self.file_id.setText(oasysgui.selectFileFromDialog(self, self.SOURCE_FILE,
                                    "Open 2-columns file with spectral power",
                                    file_extension_filter="ascii dat (*.dat *.txt *spec)"))

    def unitLabels(self):
         return ['Input beam:',
                 'From energy [eV]:',
                 'To energy [eV]:',
                 'Energy points:  ',
                 'File with input beam spectral power:',
                 'Type Monochromator',
                 'Number of reflections',
                 'Polarization',
                 'Energy Selected [eV]',
                 'Miller index h','Miller index k','Miller index l','Crystal thickness [microns]',
                 "Calculation method",
                 "XOPPY/Multilayer h5 file",
                 "Grazing angle [deg]",
                 "Dump file",
                 ]

    def unitFlags(self):
         return ['True',
                 'self.SOURCE == 1',
                 'self.SOURCE == 1',
                 'self.SOURCE == 1',
                 'self.SOURCE == 2',
                 'True',
                 'True',
                 'True',
                 'self.TYPE == 1 or self.TYPE == 2',
                 'self.TYPE == 1 or self.TYPE == 2','self.TYPE == 1 or self.TYPE == 2','self.TYPE == 1 or self.TYPE == 2',
                 'self.TYPE == 2',
                 'self.TYPE == 1 or self.TYPE == 2',
                 'self.TYPE == 3',
                 'self.TYPE == 3',
                 'True']

    def get_help_name(self):
        return 'Monochromator'

    def selectFile(self):
        self.le_source_file.setText(oasysgui.selectFileFromDialog(self, self.SOURCE_FILE, "Open Source File", file_extension_filter="*.*"))

    def acceptExchangeData(self, exchangeData):  # the same as in xpower

        self.input_spectrum = None
        self.input_script = None
        self.SOURCE = 0

        try:
            if not exchangeData is None:
                if exchangeData.get_program_name() == "XOPPY":
                    no_bandwidth = False
                    if exchangeData.get_widget_name() =="UNDULATOR_FLUX" :
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() == "BM" :
                        if exchangeData.get_content("is_log_plot") == 1:
                            raise Exception("Logaritmic X scale of Xoppy Energy distribution not supported")
                        if exchangeData.get_content("calculation_type") == 0 and exchangeData.get_content("psi") in [0,2]:
                            no_bandwidth = True
                            index_flux = 6
                        else:
                            raise Exception("Xoppy result is not a Flux vs Energy distribution integrated in Psi")
                    elif exchangeData.get_widget_name() =="XWIGGLER" :
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() =="WS" :
                        no_bandwidth = True
                        index_flux = 2
                    elif exchangeData.get_widget_name() =="XTUBES" :
                        index_flux = 1
                        no_bandwidth = True
                    elif exchangeData.get_widget_name() =="XTUBE_W" :
                        index_flux = 1
                        no_bandwidth = True
                    elif exchangeData.get_widget_name() =="BLACK_BODY" :
                        no_bandwidth = True
                        index_flux = 2

                    elif exchangeData.get_widget_name() =="UNDULATOR_RADIATION" :
                        no_bandwidth = True
                        index_flux = 1
                    elif exchangeData.get_widget_name() =="POWER" :
                        no_bandwidth = True
                        index_flux = -1
                    elif exchangeData.get_widget_name() =="POWER3D" :
                        no_bandwidth = True
                        index_flux = 1

                    else:
                        raise Exception("Xoppy Source not recognized")

                    spectrum = exchangeData.get_content("xoppy_data")

                    if exchangeData.get_widget_name() =="UNDULATOR_RADIATION" or \
                        exchangeData.get_widget_name() =="POWER3D":
                        [p, e, h, v ] = spectrum
                        tmp = p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*codata.e*1e3
                        spectrum = numpy.vstack((e,p.sum(axis=2).sum(axis=1)*(h[1]-h[0])*(v[1]-v[0])*
                                                 codata.e*1e3))
                        self.input_spectrum = spectrum
                    else:

                        if not no_bandwidth:
                            spectrum[:,index_flux] /= 0.001*spectrum[:,0]

                        self.input_spectrum = numpy.vstack((spectrum[:,0],spectrum[:,index_flux]))

                    try:
                        self.input_script = exchangeData.get_content("xoppy_script")
                    except:
                        self.input_script = None

                    self.process_showers()
                    self.compute()
                elif exchangeData.get_program_name() == "SRW":
                    if exchangeData.get_widget_name() =="UNDULATOR_SPECTRUM" :
                        spectrum = exchangeData.get_content("srw_data")

                        self.input_spectrum = numpy.vstack((spectrum[:, 0], spectrum[:, 1]))
                        self.input_script = None

                        self.process_showers()
                        self.compute()

        except Exception as exception:
            QMessageBox.critical(self, "Error", str(exception), QMessageBox.Ok)

    def check_fields(self):
        if self.TYPE  ==  1:
            self.ENER_SELECTED = congruence.checkPositiveNumber(self.ENER_SELECTED, "Energy Selected [eV]")
            self.H_MILLER = congruence.checkNumber(self.H_MILLER, "H Miller")
            self.K_MILLER = congruence.checkNumber(self.K_MILLER, "K Miller")
            self.L_MILLER = congruence.checkNumber(self.H_MILLER, "L Miller")
        if self.TYPE == 2:
            self.ENER_SELECTED = congruence.checkPositiveNumber(self.ENER_SELECTED, "Energy Selected [eV]")
            self.H_MILLER = congruence.checkNumber(self.H_MILLER, "H Miller")
            self.K_MILLER = congruence.checkNumber(self.K_MILLER, "K Miller")
            self.L_MILLER = congruence.checkNumber(self.H_MILLER, "L Miller")
            self.THICK = congruence.checkPositiveNumber(self.THICK, "Laue crystal thickness [mm]")
        if self.TYPE == 3:
            self.ENER_SELECTED = congruence.checkPositiveNumber(self.ENER_SELECTED, "Energy Selected [eV]")

        if self.SOURCE == 1:
            self.ENER_MIN = congruence.checkPositiveNumber(self.ENER_MIN, "Energy from")
            self.ENER_MAX = congruence.checkStrictlyPositiveNumber(self.ENER_MAX, "Energy to")
            congruence.checkLessThan(self.ENER_MIN, self.ENER_MAX, "Energy from", "Energy to")
            self.NPOINTS = congruence.checkStrictlyPositiveNumber(self.ENER_N, "Energy Points")
        elif self.SOURCE == 2:
            congruence.checkFile(self.SOURCE_FILE)

    def do_xoppy_calculation(self):
        if self.SOURCE == 0:
            if self.input_spectrum is None:
                raise Exception("No input beam")
            else:
                energies = self.input_spectrum[0,:].copy()
                source = self.input_spectrum[1,:].copy()
            if self.input_script is None:
                script_previous = '#\n# >> MISSING SCRIPT TO CREATE (energy, spectral_power) <<\n#\n'
            else:
                script_previous = self.input_script
        elif self.SOURCE == 1:
            energies = numpy.linspace(self.ENER_MIN,self.ENER_MAX,self.ENER_N)
            source = numpy.ones(energies.size)
            tmp = numpy.vstack( (energies,source))
            self.input_spectrum = source
            script_previous = "import numpy\nenergy = numpy.linspace(%g,%g,%d)\nspectral_power = numpy.ones(%d)\n" % \
                        (self.ENER_MIN,self.ENER_MAX,self.ENER_N,self.ENER_N)
        elif self.SOURCE == 2:  # file contains energy_eV and spectral power (W/eV)
            source_file = self.SOURCE_FILE
            try:
                tmp = numpy.loadtxt(source_file)
                energies = tmp[:,0]
                source = tmp[:,1]
                self.input_spectrum = source
                script_previous = "import numpy\ntmp = numpy.loadtxt(%s)\nenergy = tmp[:,0]\nspectral_power = tmp[:,1]\n" % \
                                (source_file)
            except:
                print("Error loading file %s "%(source_file))
                raise
        elif self.SOURCE == 3:  # file contains energy_eV and flux (ph/s/0.1%bw
            source_file = self.SOURCE_FILE
            try:
                tmp = numpy.loadtxt(source_file)
                energies = tmp[:,0]
                source = tmp[:,1] * (codata.e * 1e3)
                self.input_spectrum = source
                script_previous = "import numpy\nimport scipy.constants as codata\ntmp = numpy.loadtxt(%s)\nenergy = tmp[:,0]\nspectral_power = tmp[:,1] / (codata.e * 1e3)\n" % \
                                (source_file)
            except:
                print("Error loading file %s "%(source_file))
                raise

        out_dictionary = xoppy_calc_power_monochromator(energies, source,
                                                        TYPE                 = self.TYPE,
                                                        ENER_SELECTED        = self.ENER_SELECTED,
                                                        METHOD               = self.METHOD,
                                                        THICK                = self.THICK,
                                                        ML_H5_FILE           = self.ML_H5_FILE,
                                                        ML_GRAZING_ANGLE_DEG =self.ML_GRAZING_ANGLE_DEG,
                                                        N_REFLECTIONS        = self.N_REFLECTIONS,
                                                        FILE_DUMP            = self.FILE_DUMP,
                                                        polarization         = self.POLARIZATION,
                                                        output_file          = "monochromator.spec",
                                                        )

        print(out_dictionary["info"])

        dict_parameters = {
            "TYPE"                : self.TYPE,
            "ENER_SELECTED"       : self.ENER_SELECTED,
            "METHOD"              : self.METHOD,
            "THICK"               : self.THICK,
            "ML_H5_FILE"          : self.ML_H5_FILE,
            "ML_GRAZING_ANGLE_DEG": self.ML_GRAZING_ANGLE_DEG,
            "N_REFLECTIONS"       : self.N_REFLECTIONS,
            "FILE_DUMP"           : self.FILE_DUMP,
            "polarization"        : self.POLARIZATION,
            "output_file"         : "monochromator.spec",
        }

        script_element = self.script_template().format_map(dict_parameters)

        script = script_previous + script_element

        self.xoppy_script.set_code(script)

        return out_dictionary, script

    def script_template(self):
        return """

#
# script to make the calculations (created by XOPPY:xpower)
#

import numpy
from xoppylib.power.xoppy_calc_power_monochromator import xoppy_calc_power_monochromator
import xraylib
from dabax.dabax_xraylib import DabaxXraylib

out_dictionary = xoppy_calc_power_monochromator(
        energy, # array with energies in eV
        spectral_power, # array with source spectral density
        TYPE                 = {TYPE}, # 0=None, 1=Crystal Bragg, 2=Crystal Laue, 3=Multilayer
        ENER_SELECTED        = {ENER_SELECTED}, # Energy to set crystal monochromator
        METHOD               = {METHOD}, # For crystals, in crystalpy, 0=Zachariasem, 1=Guigay
        THICK                = {THICK}, # crystal thicknes Laur crystal in um
        ML_H5_FILE           = "{ML_H5_FILE}", # File with inputs from multilaters (from xoppy/Multilayer)
        ML_GRAZING_ANGLE_DEG = {ML_GRAZING_ANGLE_DEG}, # for multilayers the grazing angle in degrees
        N_REFLECTIONS        = {N_REFLECTIONS}, # number of reflections (crystals or multilayers)
        FILE_DUMP            = {FILE_DUMP}, # 0=No, 1=yes
        polarization         = {polarization}, # 0=sigma, 1=pi, 2=unpolarized
        output_file          = "{output_file}", # filename if FILE_DUMP=1
        )


# data to pass
energy = out_dictionary["data"][0,:]
spectral_power = out_dictionary["data"][-1,:]

#                       
# example plots
#
from srxraylib.plot.gol import plot
plot(out_dictionary["data"][0,:], out_dictionary["data"][1,:],
    out_dictionary["data"][0,:], out_dictionary["data"][-1,:],
    xtitle=out_dictionary["labels"][0],
    legend=[out_dictionary["labels"][1],out_dictionary["labels"][-1]],
    title='Spectral Power [W/eV]')

#
# end script
#
"""

    def extract_data_from_xoppy_output(self, calculation_output):
        out_dictionary, script = calculation_output

        # send exchange
        calculated_data = DataExchangeObject("XOPPY", self.get_data_exchange_widget_name())
        try:
            calculated_data.add_content("xoppy_data", out_dictionary["data"].T)
        except:
            pass

        try:
            calculated_data.add_content("plot_x_col", 0)
            calculated_data.add_content("plot_y_col", -1)
        except:
            pass
        try:
            calculated_data.add_content("xoppy_script", script)
        except:
            pass
        try:
            calculated_data.add_content("labels", out_dictionary["labels"])
        except:
            pass
        try:
            calculated_data.add_content("info", out_dictionary["info"])
        except:
            pass

        return calculated_data


    def get_data_exchange_widget_name(self):
        return "POWER"

    def getTitles(self):
        return ['Input Beam', 'Monochromator reflectivity', 'Spectral Power after Monochromator']

    def getXTitles(self):
        return ["Photon Energy [eV]", "Photon Energy [eV]", "Photon Energy [eV]"]

    def getYTitles(self):
        return ['Spectral Power [W/eV]', 'Reflectivity', 'Spectral Power [W/eV]']


    def getVariablesToPlot(self):
        return [(0, 1), (0, 2), (0, 3)]

    def getLogPlot(self):
        return [(False,False), (False, False), (False, False)]

if __name__ == "__main__":

    from oasys.widgets.exchange import DataExchangeObject


    input_data_type = "POWER"

    if input_data_type == "POWER":
        # create fake UNDULATOR_FLUX xoppy exchange data
        e = numpy.linspace(7900.0, 8100.0, 1500)
        source = e/10
        received_data = DataExchangeObject("XOPPY", "POWER")
        received_data.add_content("xoppy_data", numpy.vstack((e,e,source)).T)
        received_data.add_content("xoppy_code", "US")

    elif input_data_type == "POWER3D":
        # create unulator_radiation xoppy exchange data
        from xoppylib.sources.xoppy_undulators import xoppy_calc_undulator_radiation

        e, h, v, p, code = xoppy_calc_undulator_radiation(ELECTRONENERGY=6.04,ELECTRONENERGYSPREAD=0.001,ELECTRONCURRENT=0.2,\
                                           ELECTRONBEAMSIZEH=0.000395,ELECTRONBEAMSIZEV=9.9e-06,\
                                           ELECTRONBEAMDIVERGENCEH=1.05e-05,ELECTRONBEAMDIVERGENCEV=3.9e-06,\
                                           PERIODID=0.018,NPERIODS=222,KV=1.68,DISTANCE=30.0,
                                           SETRESONANCE=0,HARMONICNUMBER=1,
                                           GAPH=0.001,GAPV=0.001,\
                                           HSLITPOINTS=41,VSLITPOINTS=41,METHOD=0,
                                           PHOTONENERGYMIN=7000,PHOTONENERGYMAX=8100,PHOTONENERGYPOINTS=20,
                                           USEEMITTANCES=1)
        received_data = DataExchangeObject("XOPPY", "POWER3D")
        received_data = DataExchangeObject("XOPPY", "UNDULATOR_RADIATION")
        received_data.add_content("xoppy_data", [p, e, h, v])
        received_data.add_content("xoppy_code", code)


    app = QApplication(sys.argv)
    w = Monochromator()
    w.acceptExchangeData(received_data)
    w.show()
    app.exec()
    w.saveSettings()



#
# Important notice: for the moment we are using wofryimpl stuff, that should be removed if this is installed
# in OASYS1-SYNED.
#


import os, sys
import numpy

from PyQt5.QtCore import QRect, Qt
from PyQt5.QtWidgets import QApplication, QMessageBox, QScrollArea, QTableWidget, QTableWidgetItem, QHeaderView, QAbstractItemView, QWidget, QLabel, QSizePolicy
from PyQt5.QtGui import QTextCursor,QFont, QPalette, QColor, QPainter, QBrush, QPen, QPixmap
from PyQt5.QtWidgets import QLabel, QSizePolicy
from PyQt5.QtWidgets import QApplication
from PyQt5 import QtGui

from silx.gui.plot import Plot2D

import orangecanvas.resources as resources

from orangewidget import gui, widget
from orangewidget.settings import Setting

from oasys.widgets.widget import OWWidget
from oasys.widgets import gui as oasysgui
from oasys.widgets import congruence

from oasys.util.oasys_objects import OasysSurfaceData
from oasys.util.oasys_util import write_surface_file
from oasys.util.oasys_util import EmittingStream

from wofryimpl.beamline.optical_elements.refractors.lens import WOLens
from wofry.propagator.wavefront2D.generic_wavefront import GenericWavefront2D

# from orangecontrib.esrf.shadow.util.conics_from_factory_parameters import ellipsoid, hyperboloid, \
#     paraboloid_focusing, paraboloid_collimating



class OWLensSurface(OWWidget):
    name = "Lens surface creator"
    id = "Lens surface creator"
    description = "Lens surface generator"
    icon = "icons/lens_surface.png"
    author = "M Sanchez del Rio"
    maintainer_email = "srio@esrf.eu"
    priority = 20
    category = ""
    keywords = ["preprocessor", "surface", "lens"]

    outputs = [{"name": "Surface Data",
                "type": OasysSurfaceData,
                "doc": "Surface Data",
                "id": "Surface Data"},
               ]

    want_main_area = 1
    want_control_area = 1

    MAX_WIDTH = 1320
    MAX_HEIGHT = 700

    IMAGE_WIDTH = 860
    IMAGE_HEIGHT = 645

    CONTROL_AREA_WIDTH = 405
    TABS_AREA_HEIGHT = 650

    #
    # variable list
    #

    lens_profile = Setting(0)

    number_of_curved_surfaces = Setting(2)
    lens_radius = Setting(200e-6)

    surface_shape = Setting(0)
    two_d_lens = Setting(0)
    wall_thickness = Setting(10e-6)
    n_lenses = Setting(1)
    multiplicative_factor = Setting(1.0)
    aperture_shape = Setting(0)
    aperture_dimension_v = Setting(1e-3)
    aperture_dimension_h = Setting(1e-3)
    shape = Setting(1)
    radius = Setting(100e-6)

    nx = Setting(101)
    ny = Setting(101)
    semilength_x = Setting(0.001)
    semilength_y = Setting(0.001)
    filename_h5 = Setting("lens.h5")

    #
    #
    #

    tab=[]
    usage_path = os.path.join(resources.package_dirname("orangecontrib.esrf.syned.widgets.extension") , "misc", "lens_surface_usage.png")

    def __init__(self):
        super().__init__()

        geom = QApplication.desktop().availableGeometry()
        self.setGeometry(QRect(round(geom.width() * 0.05),
                               round(geom.height() * 0.05),
                               round(min(geom.width() * 0.98, self.MAX_WIDTH)),
                               round(min(geom.height() * 0.95, self.MAX_HEIGHT))))

        self.setMaximumHeight(self.geometry().height())
        self.setMaximumWidth(self.geometry().width())

        gui.separator(self.controlArea)

        tabs_setting = oasysgui.tabWidget(self.controlArea)
        tabs_setting.setFixedHeight(self.TABS_AREA_HEIGHT)
        tabs_setting.setFixedWidth(self.CONTROL_AREA_WIDTH-5)


        tab_calc = oasysgui.createTabPage(tabs_setting, "Calculate")
        tab_usa = oasysgui.createTabPage(tabs_setting, "Use of the Widget")

        #
        #-------------------- calculate
        #

        button = gui.button(tab_calc, self, "Calculate", callback=self.calculate)



        #
        # lens box
        #
        out_calc = oasysgui.widgetBox(tab_calc, "Lens Settings", addSpace=True, orientation="vertical")
        self.draw_lens_settings_box(id=out_calc)
        gui.separator(out_calc)

        #
        # --------------- MESH
        #
        out_calc = oasysgui.widgetBox(tab_calc, "Mesh Parameters", addSpace=True, orientation="vertical")

        oasysgui.lineEdit(out_calc, self, "ny", "Points in Y (tangential)",
                           labelWidth=300, valueType=int, orientation="horizontal")
        oasysgui.lineEdit(out_calc, self, "nx", "Points in X (sagittal)",
                           labelWidth=300, valueType=int, orientation="horizontal")
        oasysgui.lineEdit(out_calc, self, "semilength_y", "Half length Y [m]",
                           labelWidth=300, valueType=float, orientation="horizontal")
        oasysgui.lineEdit(out_calc, self, "semilength_x", "Half length X [m]",
                           labelWidth=300, valueType=float, orientation="horizontal")


        gui.separator(out_calc)

        #
        # --------------- FILE
        #
        out_file = oasysgui.widgetBox(tab_calc, "Output hdf5 file", addSpace=True, orientation="vertical")

        oasysgui.lineEdit(out_file , self, "filename_h5", "Output filename *.h5",
                           labelWidth=150, valueType=str, orientation="horizontal")

        gui.separator(out_file)

        #
        #-------------------- Use
        #

        tab_usa.setStyleSheet("background-color: white;")

        usage_box = oasysgui.widgetBox(tab_usa, "", addSpace=True, orientation="horizontal")

        label = QLabel("")
        label.setAlignment(Qt.AlignCenter)
        label.setSizePolicy(QSizePolicy.Expanding, QSizePolicy.Expanding)
        label.setPixmap(QPixmap(self.usage_path))

        usage_box.layout().addWidget(label)

        #
        #
        #
        gui.rubber(self.controlArea)
        self.initializeTabs()
        self.set_visible()
        gui.rubber(self.mainArea)

    def initializeTabs(self):
        self.tabs = oasysgui.tabWidget(self.mainArea)

        self.tab = [oasysgui.createTabPage(self.tabs, "Results"),
                    oasysgui.createTabPage(self.tabs, "Output"),
        ]

        for tab in self.tab:
            tab.setFixedHeight(self.IMAGE_HEIGHT)
            tab.setFixedWidth(self.IMAGE_WIDTH)

        self.plot_canvas = [None] * len(self.tab)

        # tab index 1
        self.profileInfo = oasysgui.textArea()
        profile_box = oasysgui.widgetBox(self.tab[1], "", addSpace=True, orientation="horizontal")
        profile_box.layout().addWidget(self.profileInfo)

        for index in range(len(self.tab)):
            try:
                self.tab[index].layout().addWidget(self.plot_canvas[index])
            except:
                pass
        self.tabs.setCurrentIndex(0)

    def draw_lens_settings_box(self, id):

        gui.comboBox(id, self, "surface_shape", label="Lens shape", labelWidth=350,
                     items=["Plane", "Parabolic"],
                     sendSelectedValue=False, orientation="horizontal", callback=self.set_visible)

        oasysgui.lineEdit(id, self, "wall_thickness", "(t_wall) Wall thickness [m]", labelWidth=260, valueType=float, orientation="horizontal")




        self.lens_id = oasysgui.widgetBox(id, orientation="vertical", height=None)

        gui.comboBox(self.lens_id, self, "number_of_curved_surfaces", label="Number of curved surfaces", labelWidth=350,
                     items=["0 (parallel plate)", "1 (plano-concave)", "2 (bi-concave)"],
                     sendSelectedValue=False, orientation="horizontal", callback=self.set_visible)


        oasysgui.lineEdit(self.lens_id, self, "lens_radius", "(R) radius of curvature [m]", labelWidth=260,
                          valueType=float, orientation="horizontal",)

        gui.comboBox(self.lens_id, self, "two_d_lens", label="Focusing in", labelWidth=350,
                     items=["2D", "1D (tangential)", "1D (sagittal)"], sendSelectedValue=False, orientation="horizontal")


        gui.comboBox(self.lens_id, self, "aperture_shape", label="Aperture shape", labelWidth=350,
                     items=["Circular", "Rectangular"],
                     sendSelectedValue=False, orientation="horizontal", callback=self.set_visible)

        oasysgui.lineEdit(self.lens_id, self, "aperture_dimension_v", "Aperture V (height/diameter) [m]", labelWidth=260,
                          valueType=float, orientation="horizontal",)

        self.lens_width_id = oasysgui.widgetBox(id, orientation="vertical", height=None)
        oasysgui.lineEdit(self.lens_width_id, self, "aperture_dimension_h", "Aperture H (width) [m]", labelWidth=260,
                          valueType=float, orientation="horizontal",)

        oasysgui.lineEdit(self.lens_id, self, "n_lenses", "Number of lenses",
                          labelWidth=300, valueType=int, orientation="horizontal")

        oasysgui.lineEdit(id, self, "multiplicative_factor", "Multiplicative factor for z (defailt=1.0)",
                          labelWidth=300, valueType=float, orientation="horizontal")

    def set_visible(self):
        self.lens_id.setVisible(self.surface_shape == 1)
        self.lens_width_id.setVisible(self.aperture_shape == 1)

    def check_fields(self):
        self.nx = congruence.checkStrictlyPositiveNumber(self.nx, "Points X")
        self.ny = congruence.checkStrictlyPositiveNumber(self.ny, "Points Y")
        self.semilength_x = congruence.checkStrictlyPositiveNumber(self.semilength_x, "Half length X")
        self.semilength_y = congruence.checkStrictlyPositiveNumber(self.semilength_y, "Half length Y")
        # self.source_ow = congruence.checkNumber(self.source_oe, "Distance source-mirror")
        # self.oe_image = congruence.checkNumber(self.oe_image, "Distance mirror-image")
        # self.radius = congruence.checkStrictlyPositiveNumber(self.radius, "radius of curvature at tip")
        # self.n_lenses = congruence.checkStrictlyPositiveNumber(self.n_lenses, "Number of lenses")
        # self.thickness = congruence.checkNumber(self.thickness, "Thickness")

    def writeStdOut(self, text="", initialize=False):
        cursor = self.profileInfo.textCursor()
        if initialize:
            self.profileInfo.setText(text)
        else:
            cursor.movePosition(QtGui.QTextCursor.End)
            cursor.insertText(text)

    def calculate(self):
        self.writeStdOut(initialize=True)
        sys.stdout = EmittingStream(textWritten=self.writeStdOut)

        self.check_fields()

        x = numpy.linspace(-self.semilength_x, self.semilength_x, self.nx)
        y = numpy.linspace(-self.semilength_y, self.semilength_y, self.ny)

        if self.surface_shape == 0: # plane
            Z = numpy.ones((self.nx, self.ny)) * self.wall_thickness
            title = "Cumulated lens profile [m]"

        else: # parabolic
            optical_element = WOLens.create_from_keywords(
                name='Real Lens 2D',
                number_of_curved_surfaces=self.number_of_curved_surfaces,
                two_d_lens=self.two_d_lens,
                surface_shape=0,
                wall_thickness=self.wall_thickness,
                lens_radius=self.lens_radius,
                material='', # self.get_material_name(index=self.material),
                refraction_index_delta=0.0, # self.refraction_index_delta,
                att_coefficient=0.0, # self.att_coefficient,
                n_lenses=self.n_lenses,
                aperture_shape=self.aperture_shape,
                aperture_dimension_h=self.aperture_dimension_h,
                aperture_dimension_v=self.aperture_dimension_v,)

            title = "Cumulated lens profile [m] R:%6.3f $\mu$m" % (1e6 * self.radius)

            output_wavefront = GenericWavefront2D.initialize_wavefront_from_range(x_min=-self.semilength_x, x_max=self.semilength_x,
                                                                                  y_min=-self.semilength_y, y_max=self.semilength_y,
                                                                                  number_of_points=(self.nx, self.ny))

            xx, yy, Z = optical_element.get_surface_thickness_mesh(output_wavefront)

        Z *= self.multiplicative_factor

        #
        # display results
        #
        write_surface_file(Z.T, x, y, self.filename_h5, overwrite=True)
        print("\nHDF5 file %s written to disk." % self.filename_h5)

        self.plot_data2D(Z, x, y, self.tab[0],
                         title=title,
                         xtitle="x (sagittal) [m] (%d pixels)" % x.size,
                         ytitle="y (tangential) [m] (%d pixels)" % y.size)

        self.send("Surface Data",
                  OasysSurfaceData(xx=x,
                                   yy=y,
                                   zz=Z.T,
                                   surface_data_file=self.filename_h5))


    def plot_data2D(self, data2D, dataX, dataY, canvas_widget_id, title="title", xtitle="X", ytitle="Y"):
        try:
            canvas_widget_id.layout().removeItem(canvas_widget_id.layout().itemAt(0))
        except:
            pass

        origin = (dataX[0], dataY[0])
        scale = (dataX[1] - dataX[0], dataY[1] - dataY[0])

        colormap = {"name": "temperature", "normalization": "linear",
                    "autoscale": True, "vmin": 0, "vmax": 0, "colors": 256}

        tmp = Plot2D()
        tmp.resetZoom()
        tmp.setXAxisAutoScale(True)
        tmp.setYAxisAutoScale(True)
        tmp.setGraphGrid(False)
        tmp.setKeepDataAspectRatio(True)
        tmp.yAxisInvertedAction.setVisible(False)
        tmp.setXAxisLogarithmic(False)
        tmp.setYAxisLogarithmic(False)
        tmp.getMaskAction().setVisible(False)
        tmp.getRoiAction().setVisible(False)
        tmp.getColormapAction().setVisible(True)
        tmp.setKeepDataAspectRatio(False)
        tmp.addImage(data2D.T,legend="1",scale=scale,origin=origin,colormap=colormap,replace=True)
        tmp.setActiveImage("1")
        tmp.setGraphXLabel(xtitle)
        tmp.setGraphYLabel(ytitle)
        tmp.setGraphTitle(title)

        canvas_widget_id.layout().addWidget(tmp)



if __name__ == "__main__":

    app = QApplication(sys.argv)
    w = OWLensSurface()
    w.show()
    app.exec()
    w.saveSettings()

    # x = numpy.linspace(-10e-3, 10e-3, 101)
    # y = numpy.linspace(-100e-3, 100e-3, 1001)
    #
    # Z, X, Y =  diaboloid_exact_point_to_segment(p=29.3,q=19.53,theta=4.5e-3,x=x,y=y,)
    # Z0, X, Y = diaboloid_approximated_point_to_segment(p=29.3, q=19.53, theta=4.5e-3, x=x, y=y, detrend=1)
    #
    # from srxraylib.plot.gol import plot_image, plot
    # plot_image((Z0) * 1e-6, x * 1e-3, y * 1e-3, xtitle="X/mm", ytitle="Y/mm", title="Z (approximated)/um", aspect="auto")
    # plot_image((Z) * 1e-6, x * 1e-3, y * 1e-3, xtitle="X/mm", ytitle="Y/mm", title="Z (exact)/um", aspect="auto")
    # plot_image((Z-Z0) * 1e-6, x * 1e-3, y * 1e-3, xtitle="X/mm", ytitle="Y/mm", title="Z (exact)-Z(approximated)/um", aspect="auto")
    #
    # ZZ0 = Z0[:, y.size//2]
    # ZZ  = Z[:, y.size//2]
    # plot(x, ZZ0 - ZZ0.min(),
    #      x,  ZZ - ZZ.min(),
    #      xtitle="X", ytitle="Z", legend=["Z (approximated)","Z (exact)"])
    #
    # ZZ0 = Z0[x.size//2, :]
    # ZZ  = Z[x.size//2, :]
    # plot(y,  ZZ0 - ZZ0.min(),
    #      y,   ZZ - ZZ.min(),
    #      xtitle="Y", ytitle="Z", legend=["Z (approximated)","Z (exact)"])


from syned.beamline.optical_elements.absorbers.filter import Filter

class FilterWithDensity(Filter):
    def __init__(self,
                 name="Undefined",
                 material="Be",
                 thickness=1e-3,
                 density=1.0,
                 boundary_shape=None):
        Filter.__init__(self, name=name, material=material, thickness=thickness, boundary_shape=boundary_shape)
        self._density = density

        # support text containg name of variable, help text and unit. Will be stored in self._support_dictionary
        self._add_support_text([
                    ("name", "Name", ""),  # TODO: this should be in filter!!!
                    ("density"      , "Density",    "g/cm3" ),
            ])

    def get_density(self):
        return self._density

